# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from typing import AsyncIterator

from pontos.github.api.client import GitHubAsyncREST
from pontos.github.api.errors import GitHubApiError
from pontos.github.models.packages import Package, PackageType, PackageVersion


class GitHubAsyncRESTPackages(GitHubAsyncREST):
    async def exists(
        self, organization: str, package_type: PackageType, package_name: str
    ) -> bool:
        """
        Check if a package exists

        Args:
            package: GitHub package (owner/name) to use

        Returns:
            True if the package exists

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    exists = api.packages.exists("foo")
        """
        api = f"/orgs/{organization}/packages/{package_type}/{package_name}"
        response = await self._client.get(api)
        return response.is_success

    async def package(
        self, organization: str, package_type: PackageType, package_name: str
    ) -> Package:
        """
        Get information about a package

        https://docs.github.com/en/rest/reference/packages#get-a-package-for-an-organization

        Args:
            organization: GitHub organization to use
            package_type: Type of the package to get
            package_name: Name of the package to get

        Raises:
            `httpx.HTTPStatusError`: If there was an error in the request

        Returns:
            Package information

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    package = api.packages.package(
                        organization="foo",
                        package_type="container",
                        package_name="bar",
                    ):

                        print(package.name)

        """
        api = f"/orgs/{organization}/packages/{package_type}/{package_name}"
        response = await self._client.get(api)
        response.raise_for_status()
        return Package.from_dict(response.json())

    async def packages(
        self, organization: str, package_type: str
    ) -> AsyncIterator[Package]:
        """
        Get information about organization packages

        https://docs.github.com/en/rest/reference/packages#list-packages-for-an-organization

        Args:
            organization: GitHub organization to use
            package_type: Type of the packages to list


        Raises:
            `httpx.HTTPStatusError`: If there was an error in the request

        Returns:
            An async iterator yielding packages information

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    async for package in api.packages.packages(
                        organization="foo",
                        package_type="container",
                    ):
                        print(package)
        """
        api = f"/orgs/{organization}/packages/{package_type}"
        params = {"per_page": "100"}
        async for response in self._client.get_all(api, params=params):
            response.raise_for_status()

            for package in response.json():
                yield Package.from_dict(package)

    async def package_version(
        self,
        organization: str,
        package_type: PackageType,
        package_name: str,
        version: int,
    ) -> PackageVersion:
        """
        Get information about a package version

        https://docs.github.com/en/rest/reference/packages#get-a-package-version-for-an-organization

        Args:
            organization: GitHub organization to use
            package_type: Type of the package to get
            package_name: Name of the package to get
            version: Version of the package to get

        Raises:
            `httpx.HTTPStatusError`: If there was an error in the request

        Returns:
            Package version information

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    package = api.packages.package_version(
                        organization="foo",
                        package_type="container",
                        package_name="bar",
                        version=1,
                    ):

                    print(package.version)
        """
        api = f"/orgs/{organization}/packages/{package_type}/{package_name}/versions/{version}"
        response = await self._client.get(api)
        if not response.is_success:
            raise GitHubApiError(response)
        return PackageVersion.from_dict(response.json())

    async def package_versions(
        self, organization: str, package_type: PackageType, package_name: str
    ) -> AsyncIterator[PackageVersion]:
        """
        Get information about package versions


        https://docs.github.com/en/rest/reference/packages#list-package-versions-for-an-organization


        Args:
            organization: GitHub organization to use
            package_type: Type of the package to get
            package_name: Name of the package to get

        Raises:
            `httpx.HTTPStatusError`: If there was an error in the request

        Returns:
            An async iterator yielding package versions

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    async for package in api.packages.package_versions(
                        organization="foo",
                        package_type="container",
                        package_name="bar",
                    ):
                        print(package)
        """
        api = f"/orgs/{organization}/packages/{package_type}/{package_name}/versions"
        async for response in self._client.get_all(api):
            response.raise_for_status()
            versions = response.json()
            if versions:
                for version in versions:
                    yield PackageVersion.from_dict(version)

    async def package_version_tags(
        self,
        organization: str,
        package_type: PackageType,
        package_name: str,
        version: int,
    ) -> list[str]:
        """
        Get information about package version tags

        Uses https://docs.github.com/en/rest/reference/packages#get-a-package-version-for-an-organization
        and only returns the tags

        Args:
            organization: GitHub organization to use
            package_type: Type of the package to get
            package_name: Name of the package to get
            version: Version of the package to get

        Raises:
            `httpx.HTTPStatusError`: If there was an error in the request

        Returns:
            List of tags for the package version

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    tags = api.packages.package_version_tags(
                        organization="foo",
                        package_type="container",
                        package_name="bar",
                        version=1,
                    )

                    print(tags)
        """
        api = f"/orgs/{organization}/packages/{package_type}/{package_name}/versions/{version}"
        response = await self._client.get(api)
        if not response.is_success:
            raise GitHubApiError(response)
        resp = response.json()
        return resp["metadata"]["container"]["tags"]

    async def delete_package(
        self, organization: str, package_type: PackageType, package_name: str
    ) -> None:
        """
        Delete a package

        https://docs.github.com/en/rest/reference/packages#delete-a-package-for-an-organization

        Args:
            organization: GitHub organization to use
            package_type: Type of the package to delete
            package_name: Name of the package to delete

        Raises:
            `httpx.HTTPStatusError`: If there was an error in the request

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    api.packages.delete_package(
                        organization="foo",
                        package_type="container",
                        package_name="bar",
                    )
        """
        api = f"/orgs/{organization}/packages/{package_type}/{package_name}"
        response = await self._client.delete(api)
        if not response.is_success:
            raise GitHubApiError(response)

    async def delete_package_version(
        self,
        organization: str,
        package_type: PackageType,
        package_name: str,
        version: int,
    ) -> None:
        """
        Delete a package version

        https://docs.github.com/en/rest/reference/packages#delete-a-package-version-for-an-organization

        Args:
            organization: GitHub organization to use
            package_type: Type of the package to delete
            package_name: Name of the package to delete
            version: Version of the package to delete

        Raises:
            `httpx.HTTPStatusError`: If there was an error in the request

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    api.packages.delete_package_version(
                        organization="foo",
                        package_type="container",
                        package_name="bar",
                        version=1,
                    )
        """

        api = f"/orgs/{organization}/packages/{package_type}/{package_name}/versions/{version}"
        response = await self._client.delete(api)
        if not response.is_success:
            raise GitHubApiError(response)

    async def delete_package_with_tag(
        self,
        organization: str,
        package_type: PackageType,
        package_name: str,
        tag: str,
    ) -> None:
        """
        Delete a package with a specific tag

        Args:
            organization: GitHub organization to use
            package_type: Type of the package to delete
            package_name: Name of the package to delete
            tag: Tag of the package to delete

        Raises:
            `httpx.HTTPStatusError`: If there was an error in the request

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    api.packages.delete_package_with_tag(
                        organization="foo",
                        package_type="container",
                        package_name="bar",
                        tag="latest",
                    )
        """
        async for package_version in self.package_versions(
            organization, package_type, package_name
        ):
            if tag in await self.package_version_tags(
                organization, package_type, package_name, package_version.id
            ):
                api = f"/orgs/{organization}/packages/{package_type}/{package_name}/versions/{package_version.id}"
                response = await self._client.delete(api)
                if not response.is_success:
                    raise GitHubApiError(response)



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from dataclasses import dataclass
from datetime import datetime
from typing import Optional, Union

from pontos.github.models.base import GitHubModel, User
from pontos.github.models.organization import Repository
from pontos.models import StrEnum


class AlertSort(StrEnum):
    """
    The property by which to sort the alerts
    """

    CREATED = "created"
    UPDATED = "updated"


class AlertState(StrEnum):
    """
    State of the GitHub Secrets Scanning Alert
    """

    OPEN = "open"
    RESOLVED = "resolved"


class Resolution(StrEnum):
    """
    The reason for resolving the alert
    """

    FALSE_POSITIVE = "false_positive"
    WONT_FIX = "wont_fix"
    REVOKED = "revoked"
    USED_IN_TESTS = "used_in_tests"


class LocationType(StrEnum):
    """
    Type of location
    """

    COMMIT = "commit"
    ISSUE_TITLE = "issue_title"
    ISSUE_BODY = "issue_body"
    ISSUE_COMMENT = "issue_comment"


@dataclass
class SecretScanningAlert(GitHubModel):
    """
    A GitHub Secret Scanning Alert

    Attributes:
        number: The security alert number
        url: The REST API URL of the alert resource
        html_url: The GitHub URL of the alert resource
        locations_url: The REST API URL of the code locations for this alert
        state: Sets the state of the secret scanning alert. A `resolution` must
            be provided when the state is set to `resolved`.
        created_at: The time that the alert was created
        updated_at: The time that the alert was last updated
        resolution: Required when the `state` is `resolved`
        resolved_at: The time that the alert was resolved
        resolved_by: A GitHub user who resolved the alert
        secret_type: The type of secret that secret scanning detected
        secret_type_display_name: User-friendly name for the detected secret
        secret: The secret that was detected
        repository: The GitHub repository containing the alert. It's not
            returned when requesting a specific alert
        push_protection_bypassed: Whether push protection was bypassed for the
            detected secret
        push_protection_bypassed_by: A GitHub user who bypassed the push
            protection
        push_protection_bypassed_at: The time that push protection was bypassed
        resolution_comment: The comment that was optionally added when this
            alert was closed
    """

    number: int
    url: str
    html_url: str
    locations_url: str
    state: AlertState
    secret_type: str
    secret_type_display_name: str
    secret: str
    created_at: datetime
    repository: Optional[Repository] = None
    updated_at: Optional[datetime] = None
    resolution: Optional[Resolution] = None
    resolved_at: Optional[datetime] = None
    resolved_by: Optional[User] = None
    push_protection_bypassed: Optional[bool] = None
    push_protection_bypassed_by: Optional[User] = None
    push_protection_bypassed_at: Optional[datetime] = None
    resolution_comment: Optional[str] = None


@dataclass
class CommitLocation(GitHubModel):
    """
    Represents a 'commit' secret scanning location type

    Attributes:
        path: The file path in the repository
        start_line: Line number at which the secret starts in the file
        end_line: Line number at which the secret ends in the file
        start_column: The column at which the secret starts within the start
            line
        end_column: The column at which the secret ends within the end line
        blob_sha: SHA-1 hash ID of the associated blob
        blob_url: The API URL to get the associated blob resource
        commit_sha: SHA-1 hash ID of the associated commit
        commit_url: The API URL to get the associated commit resource
    """

    path: str
    start_line: int
    end_line: int
    start_column: int
    end_column: int
    blob_sha: str
    blob_url: str
    commit_sha: str
    commit_url: str


@dataclass
class IssueTitleLocation(GitHubModel):
    """
    Represents an 'issue_title' secret scanning location type

    Attributes:
        issue_title_url: The API URL to get the issue where the secret was
            detected
    """

    issue_title_url: str


@dataclass
class IssueBodyLocation(GitHubModel):
    """
    Represents an 'issue_body' secret scanning location type

    Attributes:
        issue_body_url: The API URL to get the issue where the secret was
            detected
    """

    issue_body_url: str


@dataclass
class IssueCommentLocation(GitHubModel):
    """
    Represents an 'issue_comment' secret scanning location type

    Attributes:
        issue_comment_url: he API URL to get the issue comment where the secret
            was detected
    """

    issue_comment_url: str


@dataclass
class AlertLocation(GitHubModel):
    """
    Location where the secret was detected

    Attributes:
        type: The location type
        details: Details about the location where the secret was detected
    """

    type: LocationType
    details: Union[
        CommitLocation,
        IssueTitleLocation,
        IssueBodyLocation,
        IssueCommentLocation,
    ]



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from abc import abstractmethod
from typing import Protocol, runtime_checkable


@runtime_checkable
class SupportsStr(Protocol):
    """
    A protocol for classes supporting __str__
    """

    @abstractmethod
    def __str__(self) -> str:
        pass



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from datetime import datetime
from types import TracebackType
from typing import (
    Any,
    Iterator,
    Optional,
    Type,
)

from httpx import Timeout

from pontos.errors import PontosError
from pontos.nvd.api import (
    DEFAULT_TIMEOUT_CONFIG,
    JSON,
    NVDApi,
    NVDResults,
    Params,
    convert_camel_case,
    format_date,
    now,
)
from pontos.nvd.models.cpe_match_string import CPEMatchString

__all__ = ("CPEMatchApi",)

DEFAULT_NIST_NVD_CPE_MATCH_URL = (
    "https://services.nvd.nist.gov/rest/json/cpematch/2.0"
)
MAX_CPE_MATCHES_PER_PAGE = 500


def _result_iterator(data: JSON) -> Iterator[CPEMatchString]:
    results: list[dict[str, Any]] = data.get("match_strings", [])  # type: ignore
    return (
        CPEMatchString.from_dict(result["match_string"]) for result in results
    )


class CPEMatchApi(NVDApi):
    """
    API for querying the NIST NVD CPE match information.

    Should be used as an async context manager.

    Example:
        .. code-block:: python

            from pontos.nvd.cpe_match import CPEMatchApi

            async with CPEMatchApi() as api:
                cpe = await api.cpe_match_string(...)
    """

    def __init__(
        self,
        *,
        token: Optional[str] = None,
        timeout: Optional[Timeout] = DEFAULT_TIMEOUT_CONFIG,
        rate_limit: bool = True,
    ) -> None:
        """
        Create a new instance of the CPE API.

        Args:
            token: The API key to use. Using an API key allows to run more
                requests at the same time.
            timeout: Timeout settings for the HTTP requests
            rate_limit: Set to False to ignore rate limits. The public rate
                limit (without an API key) is 5 requests in a rolling 30 second
                window. The rate limit with an API key is 50 requests in a
                rolling 30 second window.
                See https://nvd.nist.gov/developers/start-here#divRateLimits
                Default: True.
        """
        super().__init__(
            DEFAULT_NIST_NVD_CPE_MATCH_URL,
            token=token,
            timeout=timeout,
            rate_limit=rate_limit,
        )

    def cpe_matches(
        self,
        *,
        last_modified_start_date: Optional[datetime] = None,
        last_modified_end_date: Optional[datetime] = None,
        cve_id: Optional[str] = None,
        match_string_search: Optional[str] = None,
        request_results: Optional[int] = None,
        start_index: int = 0,
        results_per_page: Optional[int] = None,
    ) -> NVDResults[CPEMatchString]:
        """
        Get all CPE matches for the provided arguments

        https://nvd.nist.gov/developers/products#divCpeMatch

        Args:
            last_modified_start_date: Return all CPE matches last modified after this date.
            last_modified_end_date: Return all CPE matches last modified before this date.
            cve_id: Return all CPE matches for this Common Vulnerabilities and Exposures identifier.
            match_string_search: Return all CPE matches that conform to the given pattern
            request_results: Number of CPE matches to download. Set to None
                (default) to download all available matches.
            start_index: Index of the first CPE match to be returned. Useful
                only for paginated requests that should not start at the first
                page.
            results_per_page: Number of results in a single requests. Mostly
                useful for paginated requests.

        Returns:
            A NVDResponse for CPE matches

        Example:
            .. code-block:: python

                from pontos.nvd.cpe_match import CPEMatchApi

                async with CPEMatchApi() as api:
                    async for match_string in api.matches(cve_id='CVE-2024-1234'):
                        print(match_string)

                    json = api.matches(cve_id='CVE-2024-1234').json()

                    async for match_strings in api.matches(
                        cve_id='CVE-2024-1234',
                    ).chunks():
                        for match_string in match_strings:
                        print(match_string)
        """
        params: Params = {}

        if last_modified_start_date:
            params["lastModStartDate"] = format_date(last_modified_start_date)
            if not last_modified_end_date:
                params["lastModEndDate"] = format_date(now())
        if last_modified_end_date:
            params["lastModEndDate"] = format_date(last_modified_end_date)

        if cve_id:
            params["cveId"] = cve_id
        if match_string_search:
            params["matchStringSearch"] = match_string_search

        results_per_page = min(
            results_per_page or MAX_CPE_MATCHES_PER_PAGE,
            request_results or MAX_CPE_MATCHES_PER_PAGE,
        )
        if start_index is None:
            start_index = 0

        return NVDResults(
            self,
            params,
            _result_iterator,
            request_results=request_results,
            results_per_page=results_per_page,
            start_index=start_index,
        )

    async def cpe_match(self, match_criteria_id: str) -> CPEMatchString:
        """
        Returns a single CPE match for the given match criteria id.

        Args:
            match_criteria_id: Match criteria identifier

        Returns:
            A CPE match for the given identifier

        Raises:
            PontosError: If match criteria ID is empty or if no match with the given ID is
                found.

        Example:
            .. code-block:: python

                from pontos.nvd.cpe_match import CVEApi

                async with CVEApi() as api:
                    match = await api.cpe_match("36FBCF0F-8CEE-474C-8A04-5075AF53FAF4")
                    print(match)
        """
        if not match_criteria_id:
            raise PontosError("Missing Match Criteria ID.")

        response = await self._get(
            params={"matchCriteriaId": str(match_criteria_id)}
        )
        response.raise_for_status()
        data = response.json(object_hook=convert_camel_case)
        match_strings = data["match_strings"]
        if not match_strings:
            raise PontosError(
                f"No match with Match Criteria ID '{match_criteria_id}' found."
            )

        match_string = match_strings[0]
        return CPEMatchString.from_dict(match_string["match_string"])

    async def __aenter__(self) -> "CPEMatchApi":
        await super().__aenter__()
        return self

    async def __aexit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_value: Optional[BaseException],
        traceback: Optional[TracebackType],
    ) -> Optional[bool]:
        return await super().__aexit__(  # type: ignore
            exc_type, exc_value, traceback
        )



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest
from unittest.mock import patch

from pontos import main
from pontos.version import __version__


class TestPontos(unittest.TestCase):
    @patch("pontos.pontos.RichTerminal")
    def test_pontos(self, terminal_mock):
        main()

        terminal_mock.return_value.print.assert_called()
        terminal_mock.return_value.indent.assert_called()
        terminal_mock.return_value.bold_info.assert_called()
        terminal_mock.return_value.info.assert_called()
        terminal_mock.return_value.warning.assert_called_once_with(
            'Use the listed commands "help" for more information '
            "and arguments description."
        )

    @patch("pontos.pontos.RichTerminal")
    @patch("sys.argv", ["pontos", "--version"])
    def test_pontos_version(self, terminal_mock):
        main()

        terminal_mock.return_value.print.assert_called_once_with(
            f"pontos version {__version__}"
        )



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

# ruff: noqa:E501

from pontos.github.api.users import GitHubAsyncRESTUsers
from tests import AsyncIteratorMock, aiter, anext
from tests.github.api import GitHubAsyncRESTTestCase, create_response


class GitHubAsyncRESTUsersTestCase(GitHubAsyncRESTTestCase):
    api_cls = GitHubAsyncRESTUsers

    async def test_users(self):
        response = create_response()
        response.json.return_value = [
            {
                "login": "octocat",
                "id": 1,
                "node_id": "MDQ6VXNlcjE=",
                "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                "gravatar_id": "",
                "url": "https://api.github.com/users/octocat",
                "html_url": "https://github.com/octocat",
                "followers_url": "https://api.github.com/users/octocat/followers",
                "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                "organizations_url": "https://api.github.com/users/octocat/orgs",
                "repos_url": "https://api.github.com/users/octocat/repos",
                "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                "received_events_url": "https://api.github.com/users/octocat/received_events",
                "type": "User",
                "site_admin": False,
            }
        ]

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.users())
        user = await anext(async_it)
        self.assertEqual(user.id, 1)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/users",
            params={"per_page": "100"},
        )

    async def test_user(self):
        response = create_response()
        response.json.return_value = {
            "login": "octocat",
            "id": 1,
            "node_id": "MDQ6VXNlcjE=",
            "avatar_url": "https://github.com/images/error/octocat_happy.gif",
            "gravatar_id": "",
            "url": "https://api.github.com/users/octocat",
            "html_url": "https://github.com/octocat",
            "followers_url": "https://api.github.com/users/octocat/followers",
            "following_url": "https://api.github.com/users/octocat/following{/other_user}",
            "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
            "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
            "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
            "organizations_url": "https://api.github.com/users/octocat/orgs",
            "repos_url": "https://api.github.com/users/octocat/repos",
            "events_url": "https://api.github.com/users/octocat/events{/privacy}",
            "received_events_url": "https://api.github.com/users/octocat/received_events",
            "type": "User",
            "site_admin": False,
            "name": "monalisa octocat",
            "company": "GitHub",
            "blog": "https://github.com/blog",
            "location": "San Francisco",
            "email": "octocat@github.com",
            "hireable": False,
            "bio": "There once was...",
            "twitter_username": "monatheoctocat",
            "public_repos": 2,
            "public_gists": 1,
            "followers": 20,
            "following": 0,
            "created_at": "2008-01-14T04:33:35Z",
            "updated_at": "2008-01-14T04:33:35Z",
        }
        self.client.get.return_value = response

        user = await self.api.user(
            "octocat",
        )

        self.client.get.assert_awaited_once_with(
            "/users/octocat",
        )

        self.assertEqual(user.id, 1)

    async def test_current_user(self):
        response = create_response()
        response.json.return_value = {
            "login": "octocat",
            "id": 1,
            "node_id": "MDQ6VXNlcjE=",
            "avatar_url": "https://github.com/images/error/octocat_happy.gif",
            "gravatar_id": "",
            "url": "https://api.github.com/users/octocat",
            "html_url": "https://github.com/octocat",
            "followers_url": "https://api.github.com/users/octocat/followers",
            "following_url": "https://api.github.com/users/octocat/following{/other_user}",
            "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
            "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
            "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
            "organizations_url": "https://api.github.com/users/octocat/orgs",
            "repos_url": "https://api.github.com/users/octocat/repos",
            "events_url": "https://api.github.com/users/octocat/events{/privacy}",
            "received_events_url": "https://api.github.com/users/octocat/received_events",
            "type": "User",
            "site_admin": False,
            "name": "monalisa octocat",
            "company": "GitHub",
            "blog": "https://github.com/blog",
            "location": "San Francisco",
            "email": "octocat@github.com",
            "hireable": False,
            "bio": "There once was...",
            "twitter_username": "monatheoctocat",
            "public_repos": 2,
            "public_gists": 1,
            "followers": 20,
            "following": 0,
            "created_at": "2008-01-14T04:33:35Z",
            "updated_at": "2008-01-14T04:33:35Z",
        }
        self.client.get.return_value = response

        user = await self.api.current_user()

        self.client.get.assert_awaited_once_with("/user")

        self.assertEqual(user.id, 1)

    async def test_user_keys(self):
        response = create_response()
        response.json.return_value = [{"id": 1, "key": "ssh-rsa AAA..."}]

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.user_keys("foo"))
        key = await anext(async_it)
        self.assertEqual(key.id, 1)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/users/foo/keys",
            params={"per_page": "100"},
        )

    async def test_keys(self):
        response = create_response()
        response.json.return_value = [{"id": 1, "key": "ssh-rsa AAA..."}]

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.keys())
        key = await anext(async_it)
        self.assertEqual(key.id, 1)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/user/keys",
            params={"per_page": "100"},
        )

    async def test_emails(self):
        response = create_response()
        response.json.return_value = [
            {
                "email": "octocat@github.com",
                "verified": True,
                "primary": True,
                "visibility": "public",
            }
        ]

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(self.api.emails())
        email = await anext(async_it)
        self.assertEqual(email.email, "octocat@github.com")

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/user/emails",
            params={"per_page": "100"},
        )

    async def test_key(self):
        response = create_response()
        response.json.return_value = {
            "key": "2Sg8iYjAxxmI2LvUXpJjkYrMxURPc8r+dB7TJyvv1234",
            "id": 2,
            "url": "https://api.github.com/user/keys/2",
            "title": "ssh-rsa AAAAB3NzaC1yc2EAAA",
            "created_at": "2020-06-11T21:31:57Z",
            "verified": False,
            "read_only": False,
        }
        self.client.get.return_value = response

        key = await self.api.key(2)

        self.client.get.assert_awaited_once_with(
            "/user/keys/2",
        )

        self.assertEqual(key.id, 2)

    async def test_delete_key(self):
        response = create_response()
        self.client.get.return_value = response

        await self.api.delete_key(2)

        self.client.delete.assert_awaited_once_with(
            "/user/keys/2",
        )

    async def test_create_key(self):
        response = create_response()
        response.json.return_value = {
            "key": "2Sg8iYjAxxmI2LvUXpJjkYrMxURPc8r+dB7TJyvv1234",
            "id": 2,
            "url": "https://api.github.com/user/keys/2",
            "title": "ssh-rsa AAAAB3NzaC1yc2EAAA",
            "created_at": "2020-06-11T21:31:57Z",
            "verified": False,
            "read_only": False,
        }
        self.client.post.return_value = response

        key = await self.api.create_key(
            "ssh-rsa AAAAB3NzaC1yc2EAAA",
            "2Sg8iYjAxxmI2LvUXpJjkYrMxURPc8r+dB7TJyvv1234",
        )

        self.client.post.assert_awaited_once_with(
            "/user/keys",
            data={
                "title": "ssh-rsa AAAAB3NzaC1yc2EAAA",
                "key": "2Sg8iYjAxxmI2LvUXpJjkYrMxURPc8r+dB7TJyvv1234",
            },
        )

        self.assertEqual(key.id, 2)



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# pylint: disable=line-too-long, arguments-differ, redefined-builtin

from datetime import datetime, timezone
from typing import Any, Dict, List, Optional
from unittest.mock import MagicMock, patch

from httpx import AsyncClient, Response

from pontos.errors import PontosError
from pontos.nvd.api import now
from pontos.nvd.cve.api import MAX_CVES_PER_PAGE, CVEApi
from pontos.nvd.models import cvss_v2, cvss_v3
from tests import AsyncMock, IsolatedAsyncioTestCase, aiter, anext
from tests.nvd import get_cve_data


def create_cve_response(
    cve_id: str,
    *,
    update: Optional[Dict[str, Any]] = None,
    results: int = 1,
) -> MagicMock:
    data = {
        "vulnerabilities": [
            {"cve": get_cve_data({"id": f"{cve_id}-{i}"})}
            for i in range(1, results + 1)
        ],
        "results_per_page": results,
    }
    if update:
        data.update(update)

    response = MagicMock(spec=Response)
    response.json.return_value = data
    return response


def create_cves_responses(
    requests: int = 2, results_per_response: int = 1
) -> List[MagicMock]:
    return [
        create_cve_response(
            f"CVE-{i}",
            update={"total_results": requests * results_per_response},
            results=results_per_response,
        )
        for i in range(1, requests + 1)
    ]


class CVEApiTestCase(IsolatedAsyncioTestCase):
    @patch("pontos.nvd.api.time.monotonic", autospec=True)
    @patch("pontos.nvd.api.AsyncClient", spec=AsyncClient)
    def setUp(
        self,
        async_client: MagicMock,
        monotonic_mock: MagicMock,
    ) -> None:
        self.http_client = AsyncMock()
        async_client.return_value = self.http_client
        monotonic_mock.return_value = 0
        self.api = CVEApi(token="token")

    async def test_no_cve_id(self):
        with self.assertRaises(PontosError):
            await self.api.cve(None)

    async def test_no_cve(self):
        data = {
            "vulnerabilities": [],
            "results_per_page": 1,
        }
        response = MagicMock(spec=Response)
        response.json.return_value = data
        self.http_client.get.return_value = response

        with self.assertRaises(PontosError):
            await self.api.cve("CVE-1")

    async def test_cve(self):
        data = {"vulnerabilities": [{"cve": get_cve_data()}]}
        response = MagicMock(spec=Response)
        response.json.return_value = data
        self.http_client.get.return_value = response

        cve = await self.api.cve("FOO-BAR")

        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={"cveId": "FOO-BAR"},
        )

        self.assertEqual(cve.id, "CVE-2022-45536")
        self.assertEqual(cve.source_identifier, "cve@mitre.org")
        self.assertEqual(
            cve.published,
            datetime(2022, 11, 22, 21, 15, 11, 103000, tzinfo=timezone.utc),
        )
        self.assertEqual(
            cve.last_modified,
            datetime(2022, 11, 23, 16, 2, 7, 367000, tzinfo=timezone.utc),
        )
        self.assertEqual(len(cve.descriptions), 1)
        self.assertEqual(len(cve.references), 2)
        self.assertEqual(len(cve.weaknesses), 0)
        self.assertEqual(len(cve.configurations), 0)
        self.assertEqual(len(cve.vendor_comments), 0)
        self.assertIsNone(cve.metrics)
        self.assertIsNone(cve.evaluator_comment)
        self.assertIsNone(cve.evaluator_solution)
        self.assertIsNone(cve.evaluator_impact)
        self.assertIsNone(cve.cisa_exploit_add)
        self.assertIsNone(cve.cisa_action_due)
        self.assertIsNone(cve.cisa_required_action)
        self.assertIsNone(cve.cisa_vulnerability_name)

    async def test_cves(self):
        self.http_client.get.side_effect = create_cves_responses()

        it = aiter(self.api.cves())
        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-1-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "resultsPerPage": MAX_CVES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-2-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={"startIndex": 1, "resultsPerPage": 1},
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    @patch("pontos.nvd.cve.api.now", spec=now)
    async def test_cves_last_modified_start_date(self, now_mock: MagicMock):
        now_mock.return_value = datetime(2022, 12, 31)
        self.http_client.get.side_effect = create_cves_responses()

        it = aiter(
            self.api.cves(last_modified_start_date=datetime(2022, 12, 1))
        )
        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-1-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "lastModStartDate": "2022-12-01T00:00:00",
                "lastModEndDate": "2022-12-31T00:00:00",
                "resultsPerPage": MAX_CVES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-2-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 1,
                "lastModStartDate": "2022-12-01T00:00:00",
                "lastModEndDate": "2022-12-31T00:00:00",
                "resultsPerPage": 1,
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cves_last_modified_end_date(self):
        self.http_client.get.side_effect = create_cves_responses()

        it = aiter(
            self.api.cves(
                last_modified_start_date=datetime(2022, 12, 1),
                last_modified_end_date=datetime(2022, 12, 31),
            )
        )
        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-1-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "lastModStartDate": "2022-12-01T00:00:00",
                "lastModEndDate": "2022-12-31T00:00:00",
                "resultsPerPage": MAX_CVES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-2-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 1,
                "lastModStartDate": "2022-12-01T00:00:00",
                "lastModEndDate": "2022-12-31T00:00:00",
                "resultsPerPage": 1,
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    @patch("pontos.nvd.cve.api.now", spec=now)
    async def test_cves_published_start_date(self, now_mock: MagicMock):
        now_mock.return_value = datetime(2022, 12, 31)
        self.http_client.get.side_effect = create_cves_responses()

        it = aiter(self.api.cves(published_start_date=datetime(2022, 12, 1)))
        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-1-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "pubStartDate": "2022-12-01T00:00:00",
                "pubEndDate": "2022-12-31T00:00:00",
                "resultsPerPage": MAX_CVES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-2-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 1,
                "pubStartDate": "2022-12-01T00:00:00",
                "pubEndDate": "2022-12-31T00:00:00",
                "resultsPerPage": 1,
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cves_published_end_date(self):
        self.http_client.get.side_effect = create_cves_responses()

        it = aiter(
            self.api.cves(
                published_start_date=datetime(2022, 12, 1),
                published_end_date=datetime(2022, 12, 31),
            )
        )
        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-1-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "pubStartDate": "2022-12-01T00:00:00",
                "pubEndDate": "2022-12-31T00:00:00",
                "resultsPerPage": MAX_CVES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-2-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 1,
                "pubStartDate": "2022-12-01T00:00:00",
                "pubEndDate": "2022-12-31T00:00:00",
                "resultsPerPage": 1,
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cves_cpe_name(self):
        self.http_client.get.side_effect = create_cves_responses()

        it = aiter(self.api.cves(cpe_name="foo-bar"))
        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-1-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "cpeName": "foo-bar",
                "resultsPerPage": MAX_CVES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-2-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 1,
                "cpeName": "foo-bar",
                "resultsPerPage": 1,
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cves_is_vulnerable(self):
        self.http_client.get.side_effect = create_cves_responses()

        it = aiter(self.api.cves(cpe_name="foo-bar", is_vulnerable=True))
        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-1-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "cpeName": "foo-bar",
                "isVulnerable": "",
                "resultsPerPage": MAX_CVES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-2-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 1,
                "cpeName": "foo-bar",
                "isVulnerable": "",
                "resultsPerPage": 1,
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cves_cvss_v2_vector(self):
        self.http_client.get.side_effect = create_cves_responses()

        it = aiter(self.api.cves(cvss_v2_vector="AV:N/AC:M/Au:N/C:N/I:P/A:N"))
        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-1-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "cvssV2Metrics": "AV:N/AC:M/Au:N/C:N/I:P/A:N",
                "resultsPerPage": MAX_CVES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-2-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 1,
                "resultsPerPage": 1,
                "cvssV2Metrics": "AV:N/AC:M/Au:N/C:N/I:P/A:N",
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cves_cvss_v3_vector(self):
        self.http_client.get.side_effect = create_cves_responses()

        it = aiter(
            self.api.cves(
                cvss_v3_vector="CVSS:3.1/AV:N/AC:L/PR:H/UI:N/S:U/C:H/I:N/A:N"
            )
        )
        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-1-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "cvssV3Metrics": "CVSS:3.1/AV:N/AC:L/PR:H/UI:N/S:U/C:H/I:N/A:N",
                "resultsPerPage": MAX_CVES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-2-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 1,
                "resultsPerPage": 1,
                "cvssV3Metrics": "CVSS:3.1/AV:N/AC:L/PR:H/UI:N/S:U/C:H/I:N/A:N",
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cves_cvss_v2_severity(self):
        self.http_client.get.side_effect = create_cves_responses()

        it = aiter(self.api.cves(cvss_v2_severity=cvss_v2.Severity.HIGH))
        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-1-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "cvssV2Severity": "HIGH",
                "resultsPerPage": MAX_CVES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-2-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 1,
                "resultsPerPage": 1,
                "cvssV2Severity": "HIGH",
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cves_cvss_v3_severity(self):
        self.http_client.get.side_effect = create_cves_responses()

        it = aiter(self.api.cves(cvss_v3_severity=cvss_v3.Severity.HIGH))
        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-1-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "cvssV3Severity": "HIGH",
                "resultsPerPage": MAX_CVES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-2-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 1,
                "resultsPerPage": 1,
                "cvssV3Severity": "HIGH",
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cves_keywords(self):
        self.http_client.get.side_effect = create_cves_responses()

        it = aiter(self.api.cves(keywords=["Mac OS X", "kernel"]))
        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-1-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "keywordSearch": "Mac OS X kernel",
                "keywordExactMatch": "",
                "resultsPerPage": MAX_CVES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-2-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 1,
                "resultsPerPage": 1,
                "keywordSearch": "Mac OS X kernel",
                "keywordExactMatch": "",
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cves_keyword(self):
        self.http_client.get.side_effect = create_cves_responses()

        it = aiter(self.api.cves(keywords="Windows"))
        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-1-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "keywordSearch": "Windows",
                "resultsPerPage": MAX_CVES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-2-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 1,
                "resultsPerPage": 1,
                "keywordSearch": "Windows",
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cves_cwe(self):
        self.http_client.get.side_effect = create_cves_responses()

        it = aiter(self.api.cves(cwe_id="CWE-1"))
        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-1-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "cweId": "CWE-1",
                "resultsPerPage": MAX_CVES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-2-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={"startIndex": 1, "resultsPerPage": 1, "cweId": "CWE-1"},
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cves_source_identifier(self):
        self.http_client.get.side_effect = create_cves_responses()

        it = aiter(self.api.cves(source_identifier="nvd@nist.gov"))
        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-1-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "sourceIdentifier": "nvd@nist.gov",
                "resultsPerPage": MAX_CVES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-2-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 1,
                "sourceIdentifier": "nvd@nist.gov",
                "resultsPerPage": 1,
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cves_virtual_match_string(self):
        self.http_client.get.side_effect = create_cves_responses()

        it = aiter(self.api.cves(virtual_match_string="foo-bar"))
        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-1-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "virtualMatchString": "foo-bar",
                "resultsPerPage": MAX_CVES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-2-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 1,
                "virtualMatchString": "foo-bar",
                "resultsPerPage": 1,
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cves_has_cert_alerts(self):
        self.http_client.get.side_effect = create_cves_responses()

        it = aiter(self.api.cves(has_cert_alerts=True))
        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-1-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "hasCertAlerts": "",
                "resultsPerPage": MAX_CVES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-2-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 1,
                "hasCertAlerts": "",
                "resultsPerPage": 1,
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cves_has_cert_notes(self):
        self.http_client.get.side_effect = create_cves_responses()

        it = aiter(self.api.cves(has_cert_notes=True))
        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-1-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "hasCertNotes": "",
                "resultsPerPage": MAX_CVES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-2-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 1,
                "hasCertNotes": "",
                "resultsPerPage": 1,
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cves_has_kev(self):
        self.http_client.get.side_effect = create_cves_responses()

        it = aiter(self.api.cves(has_kev=True))
        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-1-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "hasKev": "",
                "resultsPerPage": MAX_CVES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-2-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 1,
                "hasKev": "",
                "resultsPerPage": 1,
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cves_has_oval(self):
        self.http_client.get.side_effect = create_cves_responses()

        it = aiter(self.api.cves(has_oval=True))
        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-1-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "hasOval": "",
                "resultsPerPage": MAX_CVES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-2-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 1,
                "hasOval": "",
                "resultsPerPage": 1,
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve = await anext(it)

    async def test_cves_request_results(self):
        self.http_client.get.side_effect = create_cves_responses(
            results_per_response=2
        )

        it = aiter(self.api.cves(request_results=10))
        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-1-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "resultsPerPage": 10,
            },
        )

        self.http_client.get.reset_mock()
        cve = await anext(it)
        self.assertEqual(cve.id, "CVE-1-2")
        self.http_client.get.assert_not_called()

        self.http_client.get.reset_mock()

        cve = await anext(it)

        self.assertEqual(cve.id, "CVE-2-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cves/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 2,
                "resultsPerPage": 2,
            },
        )

        self.http_client.get.reset_mock()
        cve = await anext(it)
        self.assertEqual(cve.id, "CVE-2-2")
        self.http_client.get.assert_not_called()

        with self.assertRaises(Exception):
            cve = await anext(it)

    async def test_context_manager(self):
        async with self.api:
            pass

        self.http_client.__aenter__.assert_awaited_once()
        self.http_client.__aexit__.assert_awaited_once()

    @patch("pontos.nvd.api.time.monotonic", autospec=True)
    @patch("pontos.nvd.api.asyncio.sleep", autospec=True)
    async def test_rate_limit(
        self,
        sleep_mock: MagicMock,
        monotonic_mock: MagicMock,
    ):
        self.http_client.get.side_effect = create_cves_responses(6)
        self.api._rate_limit = 5  # pylint: disable=protected-access
        monotonic_mock.side_effect = [10.0, 11.0]

        it = aiter(self.api.cves())
        await anext(it)
        await anext(it)
        await anext(it)
        await anext(it)
        await anext(it)

        sleep_mock.assert_not_called()

        await anext(it)

        sleep_mock.assert_called_once_with(20.0)



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

"""
This script prints out all repositories existing in the space of the given
organization
"""

from argparse import ArgumentParser, Namespace

from pontos.github.api import GitHubAsyncRESTApi


def add_script_arguments(parser: ArgumentParser) -> None:
    parser.add_argument("repository", help="owner/repo")
    parser.add_argument("topics", nargs="*", help="new topics to set")


async def github_script(api: GitHubAsyncRESTApi, args: Namespace) -> int:
    if not args.topics:
        topics = await api.repositories.topics(args.repository)
    else:
        topics = await api.repositories.update_topics(
            args.repository, args.topics
        )

    for topic in topics:
        print(topic)

    return 0



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from pontos.errors import PontosError


class GitHubApiError(PontosError):
    """
    Error while using the GitHub API
    """



# SPDX-FileCopyrightText: 2021-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import asyncio
import sys

from pontos.terminal.null import NullTerminal
from pontos.terminal.rich import RichTerminal

from ._parser import parse_args


def main(args=None):
    parsed_args = parse_args(args)

    if parsed_args.quiet:
        term = NullTerminal()
    else:
        term = RichTerminal()

    term.bold_info(f"pontos-github => {parsed_args.func.__name__}")

    with term.indent():
        if not parsed_args.token:
            term.error("A Github User Token is required.")
            sys.exit(1)

        asyncio.run(parsed_args.func(term, parsed_args))


if __name__ == "__main__":
    main()



# SPDX-FileCopyrightText: 2020-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import argparse
import os
from argparse import (
    ArgumentParser,
    ArgumentTypeError,
    BooleanOptionalAction,
    Namespace,
)
from pathlib import Path
from typing import Optional, Sequence, Tuple

import shtab

from pontos.enum import enum_choice, enum_type, to_choices
from pontos.version.schemes import (
    VERSIONING_SCHEMES,
    PEP440VersioningScheme,
    VersioningScheme,
    versioning_scheme_argument_type,
)

from .create import create_release
from .helper import ReleaseType
from .show import OutputFormat, show
from .sign import sign

DEFAULT_SIGNING_KEY = "0ED1E580"


class ReleaseVersionAction(
    argparse._StoreAction
):  # pylint: disable=protected-access
    def __call__(self, parser, namespace, values, option_string=None):
        setattr(namespace, "release_type", ReleaseType.VERSION)
        setattr(namespace, self.dest, values)


def repository_type(value: str) -> str:
    """
    Validates the repository format of owner/name
    """
    splitted = value.split("/")
    if len(splitted) != 2:
        raise ArgumentTypeError(
            f"Invalid repository format {value}. Format must be owner/name."
        )
    return value


def add_create_parser(
    subparsers: argparse._SubParsersAction,
) -> None:
    create_parser: ArgumentParser = subparsers.add_parser(
        "create",
        aliases=["release"],
        help="Create a new release",
        description="Create a new release",
    )
    create_parser.set_defaults(func=create_release)
    create_parser.add_argument(
        "--versioning-scheme",
        help="Versioning scheme to use for parsing and handling version "
        f"information. Choices are {', '.join(VERSIONING_SCHEMES.keys())}. "
        "Default: %(default)s",
        default="pep440",
        type=versioning_scheme_argument_type,
    )
    create_parser.add_argument(
        "--release-type",
        help="Select the release type for calculating the release version. "
        f"Possible choices are: {to_choices(ReleaseType)}.",
        type=enum_type(ReleaseType),
        choices=enum_choice(ReleaseType),
    )
    create_parser.add_argument(
        "--release-version",
        help=(
            "Will release changelog as version. "
            "Default: lookup version in project definition."
        ),
        action=ReleaseVersionAction,
    )
    create_parser.add_argument(
        "--release-series",
        help="Create a release for a release series. Setting a release series "
        "is required if the latest tag version is newer then the to be "
        'released version. Examples: "1.2", "2", "22.4"',
    )

    next_version_group = create_parser.add_mutually_exclusive_group()

    next_version_group.add_argument(
        "--next-version",
        help=(
            "Sets the next version in project definition "
            "after the release. Default: set to next dev version"
        ),
    )

    next_version_group.add_argument(
        "--no-next-version",
        help="Don't set a next version after the release.",
        dest="next_version",
        action="store_false",
    )

    create_parser.add_argument(
        "--git-remote-name",
        help="The git remote name to push the commits and tag to",
    )
    create_parser.add_argument(
        "--git-tag-prefix",
        default="v",
        const="",
        nargs="?",
        help="Prefix for git tag versions. Default: %(default)s",
    )
    create_parser.add_argument(
        "--git-signing-key",
        help="The key to sign the commits and tag for a release",
        default=os.environ.get("GPG_SIGNING_KEY"),
    )

    create_parser.add_argument(
        "--repository",
        help="GitHub repository name (owner/name) where to publish the new "
        "release. For example octocat/Hello-World",
        type=repository_type,
    )
    create_parser.add_argument(
        "--local",
        action="store_true",
        help="Only create release changes locally and do not upload them to a "
        "remote repository. Also do not create a GitHub release.",
    )
    create_parser.add_argument(
        "--conventional-commits-config",
        dest="cc_config",
        type=Path,
        help="Conventional commits config file (toml), including conventions."
        " If not set defaults are used.",
    )
    create_parser.add_argument(
        "--update-project",
        help="Update version in project files like pyproject.toml. By default "
        "project files are updated.",
        action=BooleanOptionalAction,
        default=True,
    )
    create_parser.add_argument(
        "--github-pre-release",
        help="Enforce uploading a release as GitHub pre-release. ",
        action="store_true",
    )


def add_sign_parser(
    subparsers: argparse._SubParsersAction,
) -> None:
    sign_parser: ArgumentParser = subparsers.add_parser(
        "sign",
        help="Create signatures for an existing release",
        description="Create signatures for an existing release",
    )
    sign_parser.set_defaults(func=sign)
    sign_parser.add_argument(
        "--signing-key",
        default=DEFAULT_SIGNING_KEY,
        help="The key to sign zip, tarballs of a release. Default %(default)s.",
    )
    sign_parser.add_argument(
        "--versioning-scheme",
        help="Versioning scheme to use for parsing and handling version "
        f"information. Choices are {', '.join(VERSIONING_SCHEMES.keys())}. "
        "Default: %(default)s",
        default="pep440",
        type=versioning_scheme_argument_type,
    )
    sign_parser.add_argument(
        "--release-version",
        help="Will release changelog as version. Must be PEP 440 compliant.",
    )
    sign_parser.add_argument(
        "--release-series",
        help="Sign release files for a release series. Setting a release "
        "series is required if the latest tag version is newer then the to be "
        'signed version. Examples: "1.2", "2", "22.4"',
    )
    sign_parser.add_argument(
        "--git-tag-prefix",
        default="v",
        const="",
        nargs="?",
        help="Prefix for git tag versions. Default: %(default)s",
    )
    sign_parser.add_argument(
        "--repository",
        help="GitHub repository name (owner/name) where to download the "
        "release files from. For example octocat/Hello-World",
        type=repository_type,
    )
    sign_parser.add_argument(
        "--passphrase",
        help=(
            "Use gpg in a headless mode e.g. for "
            "the CI and use this passphrase for signing."
        ),
    )
    sign_parser.add_argument(
        "--dry-run", action="store_true", help="Do not upload signed files."
    )


def add_show_parser(
    subparsers: argparse._SubParsersAction,
) -> None:
    show_parser: ArgumentParser = subparsers.add_parser(
        "show",
        help="Show release information about the current release version and "
        "determine the next release version",
        description="Show release information about the current release "
        "version and determine the next release version",
    )
    show_parser.set_defaults(func=show)
    show_parser.add_argument(
        "--versioning-scheme",
        help="Versioning scheme to use for parsing and handling version "
        f"information. Choices are {', '.join(VERSIONING_SCHEMES.keys())}. "
        "Default: %(default)s",
        default="pep440",
        type=versioning_scheme_argument_type,
    )
    show_parser.add_argument(
        "--release-type",
        help="Select the release type for calculating the release version. "
        f"Possible choices are: {to_choices(ReleaseType)}.",
        type=enum_type(ReleaseType),
        choices=enum_choice(ReleaseType),
    )
    show_parser.add_argument(
        "--release-version",
        help=(
            "Will release changelog as version. "
            "Default: lookup version in project definition."
        ),
        action=ReleaseVersionAction,
    )
    show_parser.add_argument(
        "--release-series",
        help="Create a release for a release series. Setting a release series "
        "is required if the latest tag version is newer then the to be "
        'released version. Examples: "1.2", "2", "22.4"',
    )
    show_parser.add_argument(
        "--git-tag-prefix",
        default="v",
        const="",
        nargs="?",
        help="Prefix for git tag versions. Default: %(default)s",
    )
    show_parser.add_argument(
        "--output-format",
        help="Print in the desired output format. "
        f"Possible choices are: {to_choices(OutputFormat)}.",
        type=enum_type(OutputFormat),
        choices=enum_choice(OutputFormat),
    )


def parse_args(
    args: Optional[Sequence[str]] = None,
) -> Tuple[Optional[str], Optional[str], Namespace]:
    """
    Return user, token, parsed arguments
    """
    parser = ArgumentParser(
        description="Release handling utility.",
        prog="pontos-release",
    )
    shtab.add_argument_to(parser)

    parser.add_argument(
        "--quiet",
        "-q",
        action="store_true",
        help="Don't print messages to the terminal",
    )

    subparsers = parser.add_subparsers(
        title="subcommands",
        description="Valid subcommands",
        help="Additional help",
        dest="command",
        required=True,
    )

    add_create_parser(subparsers)
    add_sign_parser(subparsers)
    add_show_parser(subparsers)

    parsed_args = parser.parse_args(args)

    scheme: type[VersioningScheme] = getattr(
        parsed_args, "versioning_scheme", PEP440VersioningScheme
    )

    if parsed_args.func in (create_release, show):
        # check for release-type
        if not getattr(parsed_args, "release_type", None):
            parser.error("--release-type is required.")

        if (
            getattr(parsed_args, "release_version", None)
            and parsed_args.release_type != ReleaseType.VERSION
        ):
            parser.error(
                "--release-version requires --release-type "
                f"{ReleaseType.VERSION.value}"
            )

        if parsed_args.release_type == ReleaseType.VERSION and not getattr(
            parsed_args, "release_version", None
        ):
            parser.error(
                f"--release-type {ReleaseType.VERSION.value} requires to set "
                "--release-version"
            )

        next_version = getattr(parsed_args, "next_version", None)
        if next_version:
            parsed_args.next_version = scheme.parse_version(next_version)

    release_version = getattr(parsed_args, "release_version", None)
    if release_version:
        parsed_args.release_version = scheme.parse_version(release_version)

    token = os.environ.get("GITHUB_TOKEN")
    user = os.environ.get("GITHUB_USER")
    return user, token, parsed_args



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# pylint: disable=redefined-builtin, line-too-long

from pathlib import Path
from unittest.mock import MagicMock

import httpx

from pontos.github.api.artifacts import GitHubAsyncRESTArtifacts
from tests import AsyncIteratorMock, AsyncMock, aiter, anext
from tests.github.api import GitHubAsyncRESTTestCase, create_response

here = Path(__file__).parent


class GitHubAsyncRESTArtifactsTestCase(GitHubAsyncRESTTestCase):
    api_cls = GitHubAsyncRESTArtifacts

    async def test_get(self):
        response = create_response()
        response.json.return_value = {
            "id": 1,
            "node_id": "MDg6QXJ0aWZhY3QxMQ==",
            "name": "Rails",
            "size_in_bytes": 556,
            "url": "https://api.github.com/repos/octo-org/octo-docs/actions/artifacts/11",
            "archive_download_url": "https://api.github.com/repos/octo-org/octo-docs/actions/artifacts/11/zip",
            "expired": False,
            "created_at": "2020-01-10T14:59:22Z",
            "expires_at": "2020-03-21T14:59:22Z",
            "updated_at": "2020-02-21T14:59:22Z",
            "workflow_run": {
                "id": 2332938,
                "repository_id": 1296269,
                "head_repository_id": 1296269,
                "head_branch": "main",
                "head_sha": "328faa0536e6fef19753d9d91dc96a9931694ce3",
            },
        }
        self.client.get.return_value = response

        artifact = await self.api.get("foo/bar", "123")

        self.client.get.assert_awaited_once_with(
            "/repos/foo/bar/actions/artifacts/123"
        )

        self.assertEqual(artifact.id, 1)

    async def test_get_failure(self):
        response = create_response()
        self.client.get.side_effect = httpx.HTTPStatusError(
            "404", request=MagicMock(), response=response
        )

        with self.assertRaises(httpx.HTTPStatusError):
            await self.api.get("foo/bar", "123")

        self.client.get.assert_awaited_once_with(
            "/repos/foo/bar/actions/artifacts/123"
        )

    async def test_get_all(self):
        response1 = create_response()
        response1.json.return_value = {
            "artifacts": [
                {
                    "id": 1,
                    "node_id": "MDg6QXJ0aWZhY3QxMQ==",
                    "name": "Rails",
                    "size_in_bytes": 556,
                    "url": "https://api.github.com/repos/octo-org/octo-docs/actions/artifacts/11",
                    "archive_download_url": "https://api.github.com/repos/octo-org/octo-docs/actions/artifacts/11/zip",
                    "expired": False,
                    "created_at": "2020-01-10T14:59:22Z",
                    "expires_at": "2020-03-21T14:59:22Z",
                    "updated_at": "2020-02-21T14:59:22Z",
                    "workflow_run": {
                        "id": 2332938,
                        "repository_id": 1296269,
                        "head_repository_id": 1296269,
                        "head_branch": "main",
                        "head_sha": "328faa0536e6fef19753d9d91dc96a9931694ce3",
                    },
                }
            ]
        }
        response2 = create_response()
        response2.json.return_value = {
            "artifacts": [
                {
                    "id": 2,
                    "node_id": "MDg6QXJ0aWZhY3QxMw==",
                    "name": "Test output",
                    "size_in_bytes": 453,
                    "url": "https://api.github.com/repos/octo-org/octo-docs/actions/artifacts/2",
                    "archive_download_url": "https://api.github.com/repos/octo-org/octo-docs/actions/artifacts/2/zip",
                    "expired": False,
                    "created_at": "2020-01-10T14:59:22Z",
                    "expires_at": "2020-03-21T14:59:22Z",
                    "updated_at": "2020-02-21T14:59:22Z",
                    "workflow_run": {
                        "id": 2332942,
                        "repository_id": 1296269,
                        "head_repository_id": 1296269,
                        "head_branch": "main",
                        "head_sha": "178f4f6090b3fccad4a65b3e83d076a622d59652",
                    },
                },
                {
                    "id": 3,
                    "node_id": "MDg6QXJ0aWZhY3QxMw==",
                    "name": "Test output",
                    "size_in_bytes": 123,
                    "url": "https://api.github.com/repos/octo-org/octo-docs/actions/artifacts/3",
                    "archive_download_url": "https://api.github.com/repos/octo-org/octo-docs/actions/artifacts/3/zip",
                    "expired": False,
                    "created_at": "2020-01-10T14:59:22Z",
                    "expires_at": "2020-03-21T14:59:22Z",
                    "updated_at": "2020-02-21T14:59:22Z",
                    "workflow_run": {
                        "id": 2332942,
                        "repository_id": 1296269,
                        "head_repository_id": 1296269,
                        "head_branch": "main",
                        "head_sha": "178f4f6090b3fccad4a65b3e83d076a622d59652",
                    },
                },
            ]
        }

        self.client.get_all.return_value = AsyncIteratorMock(
            [response1, response2]
        )

        async_it = aiter(self.api.get_all("foo/bar"))
        artifact = await anext(async_it)
        self.assertEqual(artifact.id, 1)
        artifact = await anext(async_it)
        self.assertEqual(artifact.id, 2)
        artifact = await anext(async_it)
        self.assertEqual(artifact.id, 3)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/actions/artifacts",
            params={"per_page": "100"},
        )

    async def test_get_workflow_run_artifacts(self):
        response1 = create_response()
        response1.json.return_value = {
            "artifacts": [
                {
                    "id": 1,
                    "node_id": "MDg6QXJ0aWZhY3QxMQ==",
                    "name": "Rails",
                    "size_in_bytes": 556,
                    "url": "https://api.github.com/repos/octo-org/octo-docs/actions/artifacts/1",
                    "archive_download_url": "https://api.github.com/repos/octo-org/octo-docs/actions/artifacts/1/zip",
                    "expired": False,
                    "created_at": "2020-01-10T14:59:22Z",
                    "expires_at": "2020-03-21T14:59:22Z",
                    "updated_at": "2020-02-21T14:59:22Z",
                    "workflow_run": {
                        "id": 2332938,
                        "repository_id": 1296269,
                        "head_repository_id": 1296269,
                        "head_branch": "main",
                        "head_sha": "328faa0536e6fef19753d9d91dc96a9931694ce3",
                    },
                }
            ]
        }
        response2 = create_response()
        response2.json.return_value = {
            "artifacts": [
                {
                    "id": 2,
                    "node_id": "MDg6QXJ0aWZhY3QxMQ==",
                    "name": "Rails",
                    "size_in_bytes": 556,
                    "url": "https://api.github.com/repos/octo-org/octo-docs/actions/artifacts/2",
                    "archive_download_url": "https://api.github.com/repos/octo-org/octo-docs/actions/artifacts/2/zip",
                    "expired": False,
                    "created_at": "2020-01-10T14:59:22Z",
                    "expires_at": "2020-03-21T14:59:22Z",
                    "updated_at": "2020-02-21T14:59:22Z",
                    "workflow_run": {
                        "id": 2332938,
                        "repository_id": 1296269,
                        "head_repository_id": 1296269,
                        "head_branch": "main",
                        "head_sha": "328faa0536e6fef19753d9d91dc96a9931694ce3",
                    },
                },
                {
                    "id": 3,
                    "node_id": "MDg6QXJ0aWZhY3QxMw==",
                    "name": "Test output",
                    "size_in_bytes": 453,
                    "url": "https://api.github.com/repos/octo-org/octo-docs/actions/artifacts/3",
                    "archive_download_url": "https://api.github.com/repos/octo-org/octo-docs/actions/artifacts/3/zip",
                    "expired": False,
                    "created_at": "2020-01-10T14:59:22Z",
                    "expires_at": "2020-03-21T14:59:22Z",
                    "updated_at": "2020-02-21T14:59:22Z",
                    "workflow_run": {
                        "id": 2332942,
                        "repository_id": 1296269,
                        "head_repository_id": 1296269,
                        "head_branch": "main",
                        "head_sha": "178f4f6090b3fccad4a65b3e83d076a622d59652",
                    },
                },
            ]
        }

        self.client.get_all.return_value = AsyncIteratorMock(
            [response1, response2]
        )

        async_it = aiter(self.api.get_workflow_run_artifacts("foo/bar", "123"))
        artifact = await anext(async_it)
        self.assertEqual(artifact.id, 1)
        artifact = await anext(async_it)
        self.assertEqual(artifact.id, 2)
        artifact = await anext(async_it)
        self.assertEqual(artifact.id, 3)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/repos/foo/bar/actions/runs/123/artifacts",
            params={"per_page": "100"},
        )

    async def test_delete(self):
        response = create_response()
        self.client.delete.return_value = response

        await self.api.delete("foo/bar", "123")

        self.client.delete.assert_awaited_once_with(
            "/repos/foo/bar/actions/artifacts/123"
        )

    async def test_delete_failure(self):
        response = create_response()
        self.client.delete.side_effect = httpx.HTTPStatusError(
            "404", request=MagicMock(), response=response
        )

        with self.assertRaises(httpx.HTTPStatusError):
            await self.api.delete("foo/bar", "123")

        self.client.delete.assert_awaited_once_with(
            "/repos/foo/bar/actions/artifacts/123"
        )

    async def test_download(self):
        response = create_response(headers=MagicMock())
        response.headers.get.return_value = 2
        response.aiter_bytes.return_value = AsyncIteratorMock(["1", "2"])
        stream_context = AsyncMock()
        stream_context.__aenter__.return_value = response
        self.client.stream.return_value = stream_context

        async with self.api.download("foo/bar", 123) as download_iterable:
            it = aiter(download_iterable)
            content, progress = await anext(it)

            self.assertEqual(content, "1")
            self.assertEqual(progress, 50)

            content, progress = await anext(it)
            self.assertEqual(content, "2")
            self.assertEqual(progress, 100)

        self.client.stream.assert_called_once_with(
            "/repos/foo/bar/actions/artifacts/123/zip"
        )



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from pontos.version.commands import get_commands


class GetCommandsTestCase(unittest.TestCase):
    def test_available_commands(self):
        self.assertEqual(len(get_commands()), 6)



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from dataclasses import dataclass
from datetime import datetime
from typing import Optional

from pontos.github.models.base import GitHubModel


@dataclass
class SSHPublicKey(GitHubModel):
    """
    A public SSH key of a user

    Attributes:
        id: ID of the SSH key
        key: SSH Key
    """

    id: int
    key: str


@dataclass
class SSHPublicKeyExtended(GitHubModel):
    """
    Extended details of public SSH key of a user

    Attributes:
        id: ID of the SSH key
        key: SSH Key
        url:
        title:
        created_at
        verified:
        read_only:
    """

    id: int
    key: str
    url: str
    title: str
    created_at: datetime
    verified: bool
    read_only: bool


@dataclass
class EmailInformation(GitHubModel):
    """
    Information about an email address stored in GitHub

    Attributes:
        email: The email address
        primary: True if it is the primary email address of the user
        verified: True if the email address is verified
        visibility: public, private
    """

    email: str
    primary: bool
    verified: bool
    # visibility should be an enum but the schema didn't define the possible
    # values. therefore be safe and just use a str
    visibility: Optional[str] = None



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from pontos.models import StrEnum


class FooEnum(StrEnum):
    A = "a"
    B = "b"


class StrEnumTestCase(unittest.TestCase):
    def test_str(self):
        self.assertEqual(str(FooEnum.A), "a")
        self.assertEqual(str(FooEnum.B), "b")

    def test_str_append(self):
        self.assertEqual("say " + FooEnum.A, "say a")
        self.assertEqual("say " + FooEnum.B, "say b")

    def test_f_string(self):
        self.assertEqual(f"say {FooEnum.A}", "say a")
        self.assertEqual(f"say {FooEnum.B}", "say b")



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# pylint: disable=line-too-long, arguments-differ, redefined-builtin
# ruff: noqa: E501

from datetime import datetime
from typing import Any, Optional
from unittest.mock import MagicMock, patch
from uuid import UUID, uuid4

from httpx import AsyncClient, Response

from pontos.errors import PontosError
from pontos.nvd.api import now
from pontos.nvd.cpe_match.api import MAX_CPE_MATCHES_PER_PAGE, CPEMatchApi
from tests import AsyncMock, IsolatedAsyncioTestCase, aiter, anext
from tests.nvd import get_cpe_match_data


def uuid_replace_str(uuid: UUID, iteration: int, number: int) -> str:
    id_str = str(uuid).rsplit("-", 2)
    return f"{id_str[0]}-{iteration:04}-{number:012}"


def uuid_replace(uuid: UUID, iteration: int, number: int) -> UUID:
    return UUID(uuid_replace_str(uuid, iteration, number))


def generate_cpe_name(iteration: int, number: int) -> str:
    return f"cpe:2.3:a:acme:test-app:1.{iteration-1}.{number-1}:*:*:*:*:*:*:*"


def create_cpe_match_response(
    match_criteria_id: UUID,
    cpe_name_id: UUID,
    *,
    update: Optional[dict[str, Any]] = None,
    results: int = 1,
    iteration: int = 1,
) -> MagicMock:
    match_strings = [
        {
            "match_string": get_cpe_match_data(
                {
                    "match_criteria_id": f"{uuid_replace_str(match_criteria_id, iteration, i)}",
                    "criteria": generate_cpe_name(iteration, i),
                    "matches": [
                        {
                            "cpe_name": generate_cpe_name(iteration, i),
                            "cpe_name_id": f"{uuid_replace_str(cpe_name_id, iteration, i)}",
                        }
                    ],
                }
            )
        }
        for i in range(1, results + 1)
    ]

    data = {
        "match_strings": match_strings,
        "results_per_page": results,
    }
    if update:
        data.update(update)

    response = MagicMock(spec=Response)
    response.json.return_value = data
    return response


def create_cpe_match_responses(
    match_criteria_id: UUID,
    cpe_name_id: UUID,
    responses: int = 2,
    results_per_response: int = 1,
) -> list[MagicMock]:
    return [
        create_cpe_match_response(
            match_criteria_id=match_criteria_id,
            cpe_name_id=cpe_name_id,
            update={"total_results": responses * results_per_response},
            results=results_per_response,
            iteration=i,
        )
        for i in range(1, responses + 1)
    ]


class CPEMatchApiTestCase(IsolatedAsyncioTestCase):
    @patch("pontos.nvd.api.time.monotonic", autospec=True)
    @patch("pontos.nvd.api.AsyncClient", spec=AsyncClient)
    def setUp(self, async_client: MagicMock, monotonic_mock: MagicMock) -> None:
        self.http_client = AsyncMock()
        async_client.return_value = self.http_client
        monotonic_mock.return_value = 0
        self.api = CPEMatchApi()

    async def test_no_match_criteria_id(self):
        with self.assertRaises(PontosError):
            await self.api.cpe_match(None)

    async def test_no_match_strings(self):
        data = {
            "match_strings": [],
            "results_per_page": 1,
        }
        response = MagicMock(spec=Response)
        response.json.return_value = data
        self.http_client.get.return_value = response

        with self.assertRaises(PontosError):
            await self.api.cpe_match("DOES-NOT-EXIST")

    async def test_cpe_match(self):
        match_criteria_id = uuid_replace(uuid4(), 1, 1)
        cpe_name_id = uuid_replace(uuid4(), 1, 1)

        self.http_client.get.return_value = create_cpe_match_response(
            match_criteria_id, cpe_name_id
        )

        cpe_match_string = await self.api.cpe_match(match_criteria_id)

        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpematch/2.0",
            headers={},
            params={"matchCriteriaId": str(match_criteria_id)},
        )

        self.assertEqual(
            match_criteria_id,
            cpe_match_string.match_criteria_id,
        )
        self.assertEqual(
            generate_cpe_name(1, 1),
            cpe_match_string.criteria,
        )
        self.assertEqual(
            cpe_name_id,
            cpe_match_string.matches[0].cpe_name_id,
        )
        self.assertEqual(
            generate_cpe_name(1, 1),
            cpe_match_string.matches[0].cpe_name,
        )

    @patch("pontos.nvd.cpe_match.api.now", spec=now)
    async def test_cpe_matches_last_modified_start_date(
        self, now_mock: MagicMock
    ):
        match_criteria_id = uuid4()
        cpe_name_id = uuid4()

        now_mock.return_value = datetime(2019, 8, 30)
        self.http_client.get.side_effect = create_cpe_match_responses(
            match_criteria_id, cpe_name_id
        )

        it = aiter(
            self.api.cpe_matches(last_modified_start_date=datetime(2019, 6, 1))
        )
        cpe_match = await anext(it)

        self.assertEqual(
            uuid_replace(match_criteria_id, 1, 1), cpe_match.match_criteria_id
        )
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpematch/2.0",
            headers={},
            params={
                "startIndex": 0,
                "lastModStartDate": "2019-06-01T00:00:00",
                "lastModEndDate": "2019-08-30T00:00:00",
                "resultsPerPage": MAX_CPE_MATCHES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cpe_match = await anext(it)

        self.assertEqual(
            uuid_replace(match_criteria_id, 2, 1), cpe_match.match_criteria_id
        )
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpematch/2.0",
            headers={},
            params={
                "startIndex": 1,
                "lastModStartDate": "2019-06-01T00:00:00",
                "lastModEndDate": "2019-08-30T00:00:00",
                "resultsPerPage": 1,
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cpe_match = await anext(it)

    @patch("pontos.nvd.cpe_match.api.now", spec=now)
    async def test_cpe_matches_last_modified_end_date(
        self, now_mock: MagicMock
    ):
        match_criteria_id = uuid4()
        cpe_name_id = uuid4()

        now_mock.return_value = datetime(2019, 8, 30)
        self.http_client.get.side_effect = create_cpe_match_responses(
            match_criteria_id, cpe_name_id
        )

        it = aiter(
            self.api.cpe_matches(last_modified_end_date=datetime(2019, 8, 1))
        )
        cpe_match = await anext(it)

        self.assertEqual(
            uuid_replace(match_criteria_id, 1, 1), cpe_match.match_criteria_id
        )
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpematch/2.0",
            headers={},
            params={
                "startIndex": 0,
                "lastModEndDate": "2019-08-01T00:00:00",
                "resultsPerPage": MAX_CPE_MATCHES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cpe_match = await anext(it)

        self.assertEqual(
            uuid_replace(match_criteria_id, 2, 1), cpe_match.match_criteria_id
        )
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpematch/2.0",
            headers={},
            params={
                "startIndex": 1,
                "lastModEndDate": "2019-08-01T00:00:00",
                "resultsPerPage": 1,
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cpe_match = await anext(it)

    async def test_cpe_matches_cve_id(self):
        match_criteria_id = uuid4()
        cpe_name_id = uuid4()

        self.http_client.get.side_effect = create_cpe_match_responses(
            match_criteria_id, cpe_name_id
        )

        it = aiter(self.api.cpe_matches(cve_id="CVE-2010-3574"))
        cpe_match = await anext(it)

        self.assertEqual(
            uuid_replace(match_criteria_id, 1, 1), cpe_match.match_criteria_id
        )
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpematch/2.0",
            headers={},
            params={
                "startIndex": 0,
                "cveId": "CVE-2010-3574",
                "resultsPerPage": MAX_CPE_MATCHES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cpe_match = await anext(it)

        self.assertEqual(
            uuid_replace(match_criteria_id, 2, 1), cpe_match.match_criteria_id
        )
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpematch/2.0",
            headers={},
            params={
                "startIndex": 1,
                "cveId": "CVE-2010-3574",
                "resultsPerPage": 1,
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cpe_match = await anext(it)

    async def test_cpe_matches_match_string_search(self):
        match_criteria_id = uuid4()
        cpe_name_id = uuid4()

        self.http_client.get.side_effect = create_cpe_match_responses(
            match_criteria_id, cpe_name_id
        )

        it = aiter(
            self.api.cpe_matches(
                match_string_search="cpe:2.3:a:sun:jre:*:*:*:*:*:*:*:*"
            )
        )
        cpe_match = await anext(it)

        self.assertEqual(
            uuid_replace(match_criteria_id, 1, 1), cpe_match.match_criteria_id
        )
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpematch/2.0",
            headers={},
            params={
                "startIndex": 0,
                "matchStringSearch": "cpe:2.3:a:sun:jre:*:*:*:*:*:*:*:*",
                "resultsPerPage": MAX_CPE_MATCHES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cpe_match = await anext(it)

        self.assertEqual(
            uuid_replace(match_criteria_id, 2, 1), cpe_match.match_criteria_id
        )
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpematch/2.0",
            headers={},
            params={
                "startIndex": 1,
                "matchStringSearch": "cpe:2.3:a:sun:jre:*:*:*:*:*:*:*:*",
                "resultsPerPage": 1,
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cpe_match = await anext(it)

    async def test_cpe_matches_request_results(self):
        match_criteria_id = uuid4()
        cpe_name_id = uuid4()

        self.http_client.get.side_effect = create_cpe_match_responses(
            match_criteria_id=match_criteria_id,
            cpe_name_id=cpe_name_id,
            results_per_response=2,
        )

        it = aiter(self.api.cpe_matches(request_results=10))
        cpe_match = await anext(it)

        self.assertEqual(
            uuid_replace(match_criteria_id, 1, 1), cpe_match.match_criteria_id
        )
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpematch/2.0",
            headers={},
            params={
                "startIndex": 0,
                "resultsPerPage": 10,
            },
        )

        self.http_client.get.reset_mock()
        cpe_match = await anext(it)
        self.assertEqual(
            uuid_replace(match_criteria_id, 1, 2), cpe_match.match_criteria_id
        )
        self.http_client.get.assert_not_called()

        self.http_client.get.reset_mock()

        cpe_match = await anext(it)

        self.assertEqual(
            uuid_replace(match_criteria_id, 2, 1), cpe_match.match_criteria_id
        )
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpematch/2.0",
            headers={},
            params={
                "startIndex": 2,
                "resultsPerPage": 2,
            },
        )

        self.http_client.get.reset_mock()
        cpe_match = await anext(it)
        self.assertEqual(
            uuid_replace(match_criteria_id, 2, 2), cpe_match.match_criteria_id
        )
        self.http_client.get.assert_not_called()

        with self.assertRaises(Exception):
            cpe_match = await anext(it)

    async def test_context_manager(self):
        async with self.api:
            pass

        self.http_client.__aenter__.assert_awaited_once()
        self.http_client.__aexit__.assert_awaited_once()


class CPEMatchApiWithTokenTestCase(IsolatedAsyncioTestCase):
    @patch("pontos.nvd.api.time.monotonic", autospec=True)
    @patch("pontos.nvd.api.AsyncClient", spec=AsyncClient)
    def setUp(self, async_client: MagicMock, monotonic_mock: MagicMock) -> None:
        self.http_client = AsyncMock()
        async_client.return_value = self.http_client
        monotonic_mock.return_value = 0
        self.api = CPEMatchApi(token="token123")

    async def test_cpe_matches_request_results_with_token(self):
        match_criteria_id = uuid4()
        cpe_name_id = uuid4()

        self.http_client.get.side_effect = create_cpe_match_responses(
            match_criteria_id=match_criteria_id,
            cpe_name_id=cpe_name_id,
            results_per_response=2,
        )

        it = aiter(self.api.cpe_matches(request_results=10))
        cpe_match = await anext(it)

        self.assertEqual(
            uuid_replace(match_criteria_id, 1, 1), cpe_match.match_criteria_id
        )
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpematch/2.0",
            headers={"apiKey": "token123"},
            params={
                "startIndex": 0,
                "resultsPerPage": 10,
            },
        )

        self.http_client.get.reset_mock()
        cpe_match = await anext(it)
        self.assertEqual(
            uuid_replace(match_criteria_id, 1, 2), cpe_match.match_criteria_id
        )
        self.http_client.get.assert_not_called()

        self.http_client.get.reset_mock()

        cpe_match = await anext(it)

        self.assertEqual(
            uuid_replace(match_criteria_id, 2, 1), cpe_match.match_criteria_id
        )
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cpematch/2.0",
            headers={"apiKey": "token123"},
            params={
                "startIndex": 2,
                "resultsPerPage": 2,
            },
        )

        self.http_client.get.reset_mock()
        cpe_match = await anext(it)
        self.assertEqual(
            uuid_replace(match_criteria_id, 2, 2), cpe_match.match_criteria_id
        )
        self.http_client.get.assert_not_called()

        with self.assertRaises(Exception):
            cpe_match = await anext(it)

    @patch("pontos.nvd.api.time.monotonic", autospec=True)
    @patch("pontos.nvd.api.asyncio.sleep", autospec=True)
    async def test_rate_limit_with_token(
        self,
        sleep_mock: MagicMock,
        monotonic_mock: MagicMock,
    ):
        match_criteria_id = uuid4()
        cpe_name_id = uuid4()
        self.http_client.get.side_effect = create_cpe_match_responses(
            match_criteria_id, cpe_name_id, 8
        )
        monotonic_mock.side_effect = [10, 11]

        it = aiter(self.api.cpe_matches())
        await anext(it)
        await anext(it)
        await anext(it)
        await anext(it)
        await anext(it)

        sleep_mock.assert_not_called()

        await anext(it)

        sleep_mock.assert_not_called()



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import ArgumentParser, Namespace
from typing import Optional, Sequence

import shtab


def parse_args(args: Optional[Sequence[str]] = None) -> Namespace:
    parser = ArgumentParser()
    shtab.add_argument_to(parser)
    parser.add_argument("--token", help="API key to use for querying.")
    parser.add_argument("--cve-id", help="Get changes for a specific CVE")
    parser.add_argument(
        "--event-name", help="Get all CVE associated with a specific event name"
    )
    parser.add_argument(
        "--number", "-n", metavar="N", help="Request only N CPEs", type=int
    )
    parser.add_argument(
        "--start",
        "-s",
        help="Index of the first CPE to request.",
        type=int,
    )
    return parser.parse_args(args)



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

# ruff: noqa:E501

from pontos.github.api.billing import GitHubAsyncRESTBilling
from tests.github.api import GitHubAsyncRESTTestCase, create_response


class GitHubAsyncRESTBillingTestCase(GitHubAsyncRESTTestCase):
    api_cls = GitHubAsyncRESTBilling

    async def test_actions(self):
        response = create_response()
        response.json.return_value = {
            "total_minutes_used": 305,
            "total_paid_minutes_used": 0,
            "included_minutes": 3000,
            "minutes_used_breakdown": {
                "UBUNTU": 205,
                "MACOS": 10,
                "WINDOWS": 90,
            },
        }
        self.client.get.return_value = response

        billing = await self.api.actions("foo")

        self.client.get.assert_awaited_once_with(
            "/orgs/foo/settings/billing/actions",
        )

        self.assertEqual(billing.total_minutes_used, 305)

    async def test_packages(self):
        response = create_response()
        response.json.return_value = {
            "total_gigabytes_bandwidth_used": 50,
            "total_paid_gigabytes_bandwidth_used": 40,
            "included_gigabytes_bandwidth": 10,
        }
        self.client.get.return_value = response

        billing = await self.api.packages("foo")

        self.client.get.assert_awaited_once_with(
            "/orgs/foo/settings/billing/packages",
        )

        self.assertEqual(billing.total_gigabytes_bandwidth_used, 50)

    async def test_storage(self):
        response = create_response()
        response.json.return_value = {
            "days_left_in_billing_cycle": 20,
            "estimated_paid_storage_for_month": 15,
            "estimated_storage_for_month": 40,
        }
        self.client.get.return_value = response

        billing = await self.api.storage("foo")

        self.client.get.assert_awaited_once_with(
            "/orgs/foo/settings/billing/shared-storage",
        )

        self.assertEqual(billing.days_left_in_billing_cycle, 20)



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


from pontos.errors import PontosError


class GitHubActionsError(PontosError):
    """
    A GitHub Actions related error has occurred
    """



# SPDX-FileCopyrightText: 2020-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


import re
from typing import Iterator, Literal, Optional, Tuple, Union

from .._errors import VersionError
from .._version import Version, VersionUpdate
from ..schemes import PEP440VersioningScheme
from ._command import VersionCommand


class CMakeVersionCommand(VersionCommand):
    project_file_name = "CMakeLists.txt"

    def update_version(
        self, new_version: Version, *, force: bool = False
    ) -> VersionUpdate:
        content = self.project_file_path.read_text(encoding="utf-8")
        cmake_version_parser = CMakeVersionParser(content)

        try:
            previous_version = self.get_current_version()

            if not force and new_version == previous_version:
                return VersionUpdate(previous=previous_version, new=new_version)
        except VersionError:
            # just ignore current version and override it
            previous_version = None

        new_content = cmake_version_parser.update_version(new_version)
        self.project_file_path.write_text(new_content, encoding="utf-8")
        return VersionUpdate(
            previous=previous_version,
            new=new_version,
            changed_files=[self.project_file_path],
        )

    def get_current_version(self) -> Version:
        if not self.project_file_path.exists():
            raise VersionError(f"{self.project_file_path} not found.")

        content = self.project_file_path.read_text(encoding="utf-8")
        current_version = CMakeVersionParser(content).get_current_version()
        return self.versioning_scheme.from_version(current_version)

    def verify_version(
        self, version: Union[Literal["current"], Version, None]
    ) -> None:
        current_version = self.get_current_version()

        if not version or version == "current":
            return

        if current_version != version:
            raise VersionError(
                f"Provided version {version} does not match the "
                f"current version {current_version}."
            )


class CMakeVersionParser:
    def __init__(self, cmake_content_lines: str):
        line_no, current_version, pd_line_no, pd = self._find_version_in_cmake(
            cmake_content_lines
        )
        self._cmake_content_lines = cmake_content_lines.split("\n")
        self._version_line_number = line_no
        self._current_version = current_version
        self._project_dev_version_line_number = pd_line_no
        self._project_dev_version = pd

    # The tokenizer is used to parse and identify specific elements in CMake scripts.
    # We are interested in identifying words that represent functions, variables, and their values.
    # Specifically, we want to scan for the words 'project', 'version', 'set', 'PROJECT_DEV_VERSION',
    # and their respective values, as we need to modify them.
    __cmake_scanner = re.Scanner(  # type: ignore
        [
            (
                r"#.*",
                lambda _, token: ("comment", token),
            ),  # so that we can skip ahead
            (
                r'"[^"]*"',
                lambda _, token: ("string", token),
            ),  # so that we can verify if a value is a string value
            (
                r'"[0-9]+"',
                lambda _, token: ("number", token),
            ),  # so that we can verify if a value is numeric
            (
                r"\(",
                lambda _, token: ("open_bracket", token),
            ),  # so that we can identify function calls
            (
                r"\)",
                lambda _, token: ("close_bracket", token),
            ),  # so that we can identify end of function calls
            (
                r'[^ \t\r\n()#"]+',
                lambda _, token: ("word", token),
            ),  # so that we can identify words (identifiers)
            (
                r"\n",
                lambda _, token: ("newline", token),
            ),  # so that we can keep track of the position
            (
                r"\s+",
                lambda _, token: ("special_printable", token),
            ),  # so that we can keep track of the position
        ]
    )

    def get_current_version(self) -> Version:
        return (
            PEP440VersioningScheme.parse_version(
                f"{self._current_version}.dev1"
            )
            if self.is_dev_version()
            else PEP440VersioningScheme.parse_version(self._current_version)
        )

    def update_version(self, new_version: Version) -> str:
        if new_version.is_dev_release:
            new_version = PEP440VersioningScheme.parse_version(
                f"{str(new_version.major)}."
                f"{str(new_version.minor)}."
                f"{str(new_version.patch)}"
            )
            develop = True
        else:
            develop = False

        to_update = self._cmake_content_lines[self._version_line_number]
        updated = to_update.replace(self._current_version, str(new_version))

        self._cmake_content_lines[self._version_line_number] = updated
        self._current_version = str(new_version)

        if self._project_dev_version_line_number:
            self._cmake_content_lines[self._project_dev_version_line_number] = (
                self._cmake_content_lines[
                    self._project_dev_version_line_number
                ].replace(str(int(not develop)), str(int(develop)))
            )
            self._project_dev_version = str(int(develop))

        return "\n".join(self._cmake_content_lines)

    def _find_version_in_cmake(
        self, content: str
    ) -> Tuple[int, str, Optional[int], Optional[str]]:
        in_project = False
        in_version = False

        version_line_no: Optional[int] = None
        version: Optional[str] = None

        in_set = False
        in_project_dev_version = False

        project_dev_version_line_no: Optional[int] = None
        project_dev_version: Optional[str] = None

        for lineno, token_type, value in self._tokenize(content):
            if token_type == "word" and value == "project":
                in_project = True
            elif in_project and token_type == "word" and value == "VERSION":
                in_version = True
            elif in_version and (
                token_type == "word" or token_type == "string"
            ):
                version_line_no = lineno
                version = value
                in_project = False
                in_version = False
            elif token_type == "word" and value == "set":
                in_set = True
            elif in_set and token_type == "close_bracket":
                in_set = False
            elif (
                in_set
                and token_type == "word"
                and value == "PROJECT_DEV_VERSION"
            ):
                in_project_dev_version = True
            elif in_project_dev_version and (
                token_type == "word" or token_type == "number"
            ):
                project_dev_version_line_no = lineno
                project_dev_version = value
                in_project_dev_version = False
            elif in_project and token_type == "close_bracket":
                raise ValueError("unable to find cmake version in project.")

        if not version or version_line_no is None:
            raise ValueError("unable to find cmake version.")
        return (
            version_line_no,
            version,
            project_dev_version_line_no,
            project_dev_version,
        )

    def is_dev_version(self) -> bool:
        return (
            int(self._project_dev_version) == 1
            if self._project_dev_version
            else False
        )

    def _tokenize(  # type: ignore
        self, content: str
    ) -> Iterator[Tuple[int, str, str]]:
        toks, remainder = self.__cmake_scanner.scan(content)
        if remainder != "":
            print(f"WARNING: unrecognized cmake tokens: {remainder}")

        line_num = 0

        for tok_type, tok_contents in toks:
            line_num += tok_contents.count("\n")
            yield line_num, tok_type, tok_contents.strip()



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import asyncio
import importlib
import os
from argparse import ArgumentParser, Namespace
from contextlib import contextmanager
from pathlib import Path
from types import ModuleType
from typing import Generator, Union

from httpx import Timeout

from pontos.github.api.api import GitHubAsyncRESTApi
from pontos.github.script.errors import GitHubScriptError
from pontos.helper import add_sys_path, ensure_unload_module

GITHUB_SCRIPT_FUNCTION_NAME = "github_script"
GITHUB_SCRIPT_PARSER_FUNCTION_NAME = "add_script_arguments"


@contextmanager
def load_script(
    script: Union[str, os.PathLike]
) -> Generator[ModuleType, None, None]:
    """
    A context manager to load a script module.

    The script is unloaded when the context manager exits.

    Args:
        script: Name or path of the script module to load

    Example:
        .. code-block:: python

            from pontos.github.script import load_script

            with load_script("path/to/script.py") as module:
                module.func()

            with load_script("some.python.module") as module:
                module.func()
    """
    path = Path(script)
    if path.suffix == ".py":
        module_name = path.stem

        with (
            add_sys_path(path.parent.absolute()),
            ensure_unload_module(module_name),
        ):
            yield importlib.import_module(module_name)
    else:
        module_name = str(path)
        with ensure_unload_module(module_name):
            yield importlib.import_module(module_name)


def run_github_script_function(
    module: ModuleType, token: str, timeout: float, args: Namespace
) -> int:
    """
    Run a github_script function from a Python module

    Args:
        module: Module that the GitHub script function contains
        token: A GitHub token for authentication
        timeout: Timeout for the GitHub requests in seconds
        args: Arguments forwarded to the script function

    Raises:
        GitHubScriptError: If the module doesn't have a github_script function
            or if the github_script function is not an async coroutine.

    Returns:
        The return value of the github_script coroutine

    Example:
        .. code-block:: python

            from pontos.github.script import (
                load_script,
                run_github_script_function,
            )

            with load_script("path/to/script.py") as module:
                return run_github_script_function(module, token, 60.0, args)

            with load_script("some.python.module") as module:
                return run_github_script_function(module, token, 60.0, args)
    """
    if not hasattr(module, GITHUB_SCRIPT_FUNCTION_NAME):
        raise GitHubScriptError(
            f"{module.__file__} is not a valid Pontos GitHub Script. A "
            f"{GITHUB_SCRIPT_FUNCTION_NAME} function is missing."
        )
    func = getattr(module, GITHUB_SCRIPT_FUNCTION_NAME)
    if not asyncio.iscoroutinefunction(func):
        # it's not async
        raise GitHubScriptError(
            f"{module.__file__} is not a valid Pontos GitHub Script. "
            f"{GITHUB_SCRIPT_FUNCTION_NAME} need to be an async coroutine "
            "function."
        )

    async def run_async() -> int:
        async with GitHubAsyncRESTApi(token, timeout=Timeout(timeout)) as api:
            return await func(api, args)

    loop_owner = False
    try:
        loop = asyncio.get_running_loop()
    except RuntimeError:
        loop_owner = True
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)

    try:
        retval = loop.run_until_complete(run_async())
    finally:
        if loop_owner:
            loop.close()
    return retval


def run_add_arguments_function(
    module: ModuleType, parser: ArgumentParser
) -> None:
    """
    Run a GitHub script add_script_arguments function (if available in the
    module).

    Args:
        module: Module containing the GitHub script add_script_arguments
            function
        parser: An ArgumentParser to add additional CLI arguments

    Example:
        .. code-block:: python

            from argparse import ArgumentParser
            from pontos.github.script import (
                load_script,
                run_github_script_function,
            )

            parser = ArgumentParser()

            with load_script("path/to/script.py") as module:
                run_add_arguments_function(module, parser)

            with load_script("some.python.module") as module:
                run_add_arguments_function(module, parser)
    """
    func = getattr(module, GITHUB_SCRIPT_PARSER_FUNCTION_NAME, None)
    if func:
        func(parser)



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

"""
This script prints out all repositories existing in the space of the given
organization
"""

from argparse import ArgumentParser, Namespace
from typing import Union

from rich.console import Console
from rich.table import Table

from pontos.github.api import GitHubAsyncRESTApi
from pontos.github.models.organization import RepositoryType


def repository_type(value: Union[str, RepositoryType]) -> RepositoryType:
    if isinstance(value, RepositoryType):
        return value
    return RepositoryType[value.upper()]


def add_script_arguments(parser: ArgumentParser) -> None:
    parser.add_argument("organization")

    parser.add_argument(
        "-t",
        "--type",
        type=repository_type,
        help=f"What type of repositories should be printed? Choices: "
        f"{', '.join([f.name for f in RepositoryType])}. Default: %(default)s",
        default=RepositoryType.ALL.name,
    )


async def github_script(api: GitHubAsyncRESTApi, args: Namespace) -> int:
    table = Table()
    table.add_column("Name")
    table.add_column("Description")
    table.add_column("URL")
    table.add_column("Visibility")

    repo_count = 0
    async for repo in api.organizations.get_repositories(
        args.organization, repository_type=args.type
    ):
        table.add_row(
            repo.name,
            repo.description,
            f"[link={repo.html_url}]{repo.html_url}[/link]",
            repo.visibility,
        )

        repo_count += 1

    console = Console()
    console.print(table)

    print(f"{repo_count} repositories.")
    return 0



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# pylint: disable=no-member

from argparse import Namespace
from pathlib import Path
from unittest.mock import MagicMock, call, patch

from pontos.github.cmds import (
    create_pull_request,
    create_release,
    create_tag,
    file_status,
    labels,
    repos,
    update_pull_request,
)
from pontos.github.models.base import FileStatus
from pontos.github.models.organization import RepositoryType
from pontos.testing import temp_file
from tests import AsyncIteratorMock, IsolatedAsyncioTestCase

here = Path(__file__).parent


class TestCmds(IsolatedAsyncioTestCase):
    @patch("pontos.github.api.api.GitHubAsyncRESTPullRequests", spec=True)
    async def test_file_status(self, api_mock):
        terminal = MagicMock()
        api_mock.return_value.exists.return_value = True
        api_mock.return_value.files.return_value = {
            FileStatus.ADDED: [Path("tests/github/foo/bar")],
            FileStatus.MODIFIED: [
                Path("tests/github/bar/baz"),
                Path("tests/github/baz/boo"),
            ],
        }

        with (
            temp_file(name="some.file") as test_file,
            test_file.open("w", encoding="utf8") as output,
        ):
            args = Namespace(
                repo="foo/bar",
                pull_request=8,
                output=output,
                status=[FileStatus.ADDED, FileStatus.MODIFIED],
                token="GITHUB_TOKEN",
            )

            await file_status(terminal, args)

            api_mock.return_value.exists.assert_awaited_once_with(
                repo="foo/bar", pull_request=8
            )
            api_mock.return_value.files.assert_awaited_once_with(
                repo="foo/bar",
                pull_request=8,
                status_list=[FileStatus.ADDED, FileStatus.MODIFIED],
            )

            output.flush()

            content = test_file.read_text(encoding="utf-8")
            self.assertEqual(
                content, f"{here}/foo/bar\n{here}/bar/baz\n{here}/baz/boo\n"
            )

    @patch("pontos.github.api.api.GitHubAsyncRESTReleases", spec=True)
    async def test_create_release_no_tag(self, api_mock):
        terminal = MagicMock()
        api_mock.return_value.exists.return_value = True

        args = Namespace(
            repo="foo/bar",
            tag="test_tag",
            name="test_release",
            body=None,
            target_commitish=None,
            draft=False,
            prerelease=False,
            token="GITHUB_TOKEN",
        )

        with self.assertRaises(SystemExit) as syse:
            await create_release(terminal, args)

        self.assertEqual(syse.exception.code, 1)

        api_mock.return_value.exists.assert_awaited_once_with(
            repo="foo/bar", tag="test_tag"
        )

    @patch("pontos.github.api.api.GitHubAsyncRESTReleases", spec=True)
    async def test_create_release(self, api_mock):
        terminal = MagicMock()
        api_mock.return_value.exists.return_value = False
        api_mock.return_value.create.return_value = True

        args = Namespace(
            repo="foo/bar",
            tag="test_tag",
            name="test_release",
            body=None,
            target_commitish=None,
            draft=False,
            prerelease=False,
            token="GITHUB_TOKEN",
        )

        await create_release(terminal, args)

        api_mock.return_value.exists.assert_awaited_once_with(
            repo="foo/bar", tag="test_tag"
        )
        api_mock.return_value.create.assert_awaited_once_with(
            repo="foo/bar",
            tag="test_tag",
            body=None,
            name="test_release",
            target_commitish=None,
            draft=False,
            prerelease=False,
        )

    @patch("pontos.github.api.api.GitHubAsyncRESTTags", spec=True)
    async def test_create_tag(self, api_mock):
        terminal = MagicMock()
        api_mock.return_value.create.return_value = MagicMock(sha="123")
        api_mock.return_value.create_tag_reference.return_value = True

        args = Namespace(
            repo="foo/bar",
            tag="test_tag",
            name="test_release",
            message="test msg",
            git_object="commit",
            git_object_type=None,
            email="test@test.test",
            date=None,
            token="GITHUB_TOKEN",
        )

        await create_tag(terminal, args)

        api_mock.return_value.create.assert_awaited_once_with(
            repo="foo/bar",
            tag="test_tag",
            message="test msg",
            git_object="commit",
            name="test_release",
            email="test@test.test",
            git_object_type=None,
            date=None,
        )
        api_mock.return_value.create_tag_reference.assert_awaited_once_with(
            repo="foo/bar", tag="test_tag", sha="123"
        )

    @patch("pontos.github.api.api.GitHubAsyncRESTBranches", spec=True)
    @patch("pontos.github.api.api.GitHubAsyncRESTPullRequests", spec=True)
    async def test_create_pull_request(
        self,
        pulls_api_mock,
        branches_api_mock,
    ):
        terminal = MagicMock()
        branches_api_mock.return_value.exists.return_value = [False, False]
        pulls_api_mock.return_value.create.return_value = True

        args = Namespace(
            repo="foo/bar",
            head="some-branch",
            target="main",
            title="foo",
            body="foo bar",
            token="GITHUB_TOKEN",
        )

        await create_pull_request(terminal, args)

        branches_api_mock.return_value.exists.assert_has_awaits(
            [
                call(repo="foo/bar", branch="some-branch"),
                call(repo="foo/bar", branch="main"),
            ]
        )

        pulls_api_mock.return_value.create.assert_awaited_once_with(
            repo="foo/bar",
            head_branch="some-branch",
            base_branch="main",
            title="foo",
            body="foo bar",
        )

    @patch("pontos.github.api.api.GitHubAsyncRESTBranches", spec=True)
    @patch("pontos.github.api.api.GitHubAsyncRESTPullRequests", spec=True)
    async def test_update_pull_request(
        self,
        pulls_api_mock,
        branches_api_mock,
    ):
        terminal = MagicMock()
        branches_api_mock.return_value.exists.return_value = True
        pulls_api_mock.return_value.create.return_value = True

        args = Namespace(
            repo="foo/bar",
            target="main",
            pull_request=9,
            title="foo",
            body="foo bar",
            token="GITHUB_TOKEN",
        )

        await update_pull_request(terminal, args)

        branches_api_mock.return_value.exists.assert_awaited_once_with(
            repo="foo/bar", branch="main"
        )

        pulls_api_mock.return_value.update.assert_awaited_once_with(
            repo="foo/bar",
            pull_request=9,
            base_branch="main",
            title="foo",
            body="foo bar",
        )

    @patch("pontos.github.api.api.GitHubAsyncRESTPullRequests", spec=True)
    @patch("pontos.github.api.api.GitHubAsyncRESTLabels", spec=True)
    async def test_labels(
        self,
        labels_api_mock,
        pulls_api_mock,
    ):
        terminal = MagicMock()
        pulls_api_mock.return_value.exists.return_value = True
        labels_api_mock.return_value.get_all.return_value = AsyncIteratorMock(
            ["foo", "bar"]
        )

        args = Namespace(
            repo="foo/bar",
            issue=9,
            labels=["baz"],
            token="GITHUB_TOKEN",
        )

        await labels(terminal, args)

        pulls_api_mock.return_value.exists.assert_awaited_once_with(
            repo="foo/bar", pull_request=9
        )

        labels_api_mock.return_value.get_all.assert_called_once_with(
            repo="foo/bar",
            issue=9,
        )

        labels_api_mock.return_value.set_all.assert_awaited_once_with(
            repo="foo/bar", issue=9, labels=["foo", "bar", "baz"]
        )

    @patch("pontos.github.api.api.GitHubAsyncRESTOrganizations", spec=True)
    async def test_repos(self, api_mock):
        terminal = MagicMock()
        api_mock.return_value.exists.return_value = True
        api_mock.return_value.get_repositories.return_value = AsyncIteratorMock(
            ["repo1", "repo2"]
        )

        args = Namespace(
            orga="foo",
            repo="bar",
            path=None,
            type=RepositoryType.PUBLIC,
            token="GITHUB_TOKEN",
        )

        await repos(terminal, args)

        api_mock.return_value.exists.assert_awaited_once_with("foo")
        api_mock.return_value.get_repositories.assert_called_once_with(
            organization="foo", repository_type=RepositoryType.PUBLIC
        )



# SPDX-FileCopyrightText: 2020-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import json
import re
from pathlib import Path
from typing import Any, Dict, List, Literal, Union

from .._errors import VersionError
from .._version import Version, VersionUpdate
from ._command import VersionCommand


# This class is used for Java Version command(s)
class JavaVersionCommand(VersionCommand):
    VERSION_PATTERN = (
        r"^(?P<pre>.*[^\d])?"
        r"("
        r"?P<version>\d+\.\d+\.\d+"
        r"([-\.]+(dev|rc|beta|a|alpha|b)\d+)*"
        r")"
        r"(?P<post>.*$)"
    )

    project_file_name = "upgradeVersion.json"

    def get_current_version(self) -> Version:
        file_versions = self._read_versions_from_files()

        last_version = self._verify_version(file_versions)

        if last_version == "":
            raise VersionError("no version found")

        return self.versioning_scheme.parse_version(last_version)

    def verify_version(
        self, version: Union[Literal["current"], Version, None]
    ) -> None:
        file_versions = self._read_versions_from_files()

        last_version = self._verify_version(file_versions)

        if last_version != str(version):
            raise VersionError(
                f"Provided version {version} does not match the "
                f"current version {last_version} "
                f"in '{self.project_file_path}'"
            )

    def update_version(
        self, new_version: Version, *, force: bool = False
    ) -> VersionUpdate:
        try:
            current_version = self.get_current_version()
            if not force and new_version == current_version:
                return VersionUpdate(previous=current_version, new=new_version)
        except VersionError:
            # just ignore current version and override it
            current_version = None

        changed_files = self._update_version_files(new_version)

        return VersionUpdate(
            previous=current_version,
            new=new_version,
            changed_files=changed_files,
        )

    def parse_line(self, version_line: str):
        return re.match(self.VERSION_PATTERN, version_line, re.DOTALL)

    def _update_version_files(self, new_version) -> List[Path]:
        config = self._load_config()

        changed_files: List[Path] = []
        for file_config in config["files"]:
            file_path = file_config["path"]
            with (Path.cwd() / file_path).open("r") as input_file_handle:
                lines = input_file_handle.readlines()
                line_number = file_config["line"]
                version_line = lines[line_number - 1]

                matches = self.parse_line(version_line)
                if matches is None:
                    raise VersionError(
                        f"Line has no version, "
                        f"file:'{file_path}' "
                        f"lineNo:{line_number} "
                        f"content:'{version_line}'"
                    )
                lines[line_number - 1] = (
                    matches.group("pre")
                    + str(new_version)
                    + matches.group("post")
                )

                content = "".join(lines)
                with (Path.cwd() / file_path).open("w") as output_file_handle:
                    output_file_handle.write(content)
                changed_files.append(Path(file_config["path"]))
        return changed_files

    def _load_config(self) -> Dict[str, Any]:
        version_config_file = Path.cwd() / "upgradeVersion.json"
        if not version_config_file.exists():
            raise VersionError(
                f"No {version_config_file} config file found. "
                "This file is required for pontos"
            )

        with version_config_file.open("r") as f:
            json_string = f.read()
            config = json.loads(json_string)
            return config

    def _read_versions_from_files(self) -> Dict[str, str]:
        config = self._load_config()

        file_versions = {}
        for file_config in config["files"]:
            file_path = file_config["path"]
            file = Path.cwd() / file_path
            if not file.exists():
                raise VersionError(f"No {file} file found.")

            with file.open("r") as f:
                line_number = file_config["line"]
                readlines = f.readlines()
                if line_number - 1 > len(readlines):
                    raise VersionError(
                        f"Line number:{line_number} "
                        f"is beyond file lines:{len(readlines) + 1} "
                        f"file:'{file_path}'"
                    )
                version_line = readlines[line_number - 1]
                matches = self.parse_line(version_line)
                if matches is None:
                    raise VersionError(
                        f"Line has no version, "
                        f"file:'{file_path}' "
                        f"lineNo:{line_number} "
                        f"content:'{version_line}'"
                    )
                file_versions[file_path] = matches.group("version")
        return file_versions

    def _verify_version(self, file_versions: Dict[str, str]) -> str:
        last_version = ""
        last_file_name = ""
        for file_name, version in file_versions.items():
            if last_version == "":
                last_version = version
                last_file_name = file_name
                continue

            if last_version != version:
                raise VersionError(
                    f"Versions are not the same "
                    f"last_file_name:'{last_file_name}' "
                    f"last_version:'{last_version}' "
                    f"file_name:'{file_name}' "
                    f"version:'{version}'"
                )
        return last_version



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import asyncio
from argparse import Namespace

from pontos.nvd.cve_changes.api import CVEChangesApi

from ._parser import parse_args

__all__ = ("CVEChangesApi",)


async def query_changes(args: Namespace) -> None:
    async with CVEChangesApi(token=args.token) as api:
        async for cve in api.changes(
            cve_id=args.cve_id,
            event_name=args.event_name,
            request_results=args.number,
            start_index=args.start,
        ):
            print(cve)


def main() -> None:
    try:
        args = parse_args()
        asyncio.run(query_changes(args))
    except KeyboardInterrupt:
        pass


if __name__ == "__main__":
    main()



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from typing import Iterable, Tuple, Type

from ._cargo import CargoVersionCommand
from ._cmake import CMakeVersionCommand
from ._command import VersionCommand
from ._go import GoVersionCommand
from ._java import JavaVersionCommand
from ._javascript import JavaScriptVersionCommand
from ._python import PythonVersionCommand

__all__ = (
    "VersionCommand",
    "CMakeVersionCommand",
    "GoVersionCommand",
    "JavaScriptVersionCommand",
    "JavaVersionCommand",
    "PythonVersionCommand",
    "CargoVersionCommand",
    "get_commands",
)

__COMMANDS: Tuple[Type[VersionCommand]] = (  # type: ignore[assignment]
    CMakeVersionCommand,
    GoVersionCommand,
    JavaVersionCommand,
    JavaScriptVersionCommand,
    PythonVersionCommand,
    CargoVersionCommand,
)


def get_commands() -> Iterable[Type[VersionCommand]]:
    """
    Returns the available VersionCommands
    """
    return __COMMANDS



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

# ruff: noqa:E501

import unittest
from datetime import datetime, timezone

from pontos.github.models.code_scanning import (
    AlertState,
    Analysis,
    CodeQLDatabase,
    CodeScanningAlert,
    DefaultSetup,
    DefaultSetupState,
    Instance,
    Language,
    Location,
    QuerySuite,
    Rule,
    SarifProcessingStatus,
    SarifUploadInformation,
    Severity,
    Tool,
)

ALERT = {
    "number": 3,
    "created_at": "2020-02-13T12:29:18Z",
    "url": "https://api.github.com/repos/octocat/hello-world/code-scanning/alerts/3",
    "html_url": "https://github.com/octocat/hello-world/code-scanning/3",
    "state": "dismissed",
    "dismissed_by": {
        "login": "octocat",
        "id": 1,
        "node_id": "MDQ6VXNlcjE=",
        "avatar_url": "https://github.com/images/error/octocat_happy.gif",
        "gravatar_id": "",
        "url": "https://api.github.com/users/octocat",
        "html_url": "https://github.com/octocat",
        "followers_url": "https://api.github.com/users/octocat/followers",
        "following_url": "https://api.github.com/users/octocat/following{/other_user}",
        "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
        "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
        "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
        "organizations_url": "https://api.github.com/users/octocat/orgs",
        "repos_url": "https://api.github.com/users/octocat/repos",
        "events_url": "https://api.github.com/users/octocat/events{/privacy}",
        "received_events_url": "https://api.github.com/users/octocat/received_events",
        "type": "User",
        "site_admin": False,
    },
    "dismissed_at": "2020-02-14T12:29:18Z",
    "dismissed_reason": "false positive",
    "dismissed_comment": "This alert is not actually correct, because there's "
    "a sanitizer included in the library.",
    "rule": {
        "id": "js/zipslip",
        "severity": "error",
        "tags": ["security", "external/cwe/cwe-022"],
        "description": "Arbitrary file write during zip extraction",
        "name": "js/zipslip",
    },
    "tool": {"name": "CodeQL", "guid": None, "version": "2.4.0"},
    "most_recent_instance": {
        "ref": "refs/heads/main",
        "analysis_key": ".github/workflows/codeql-analysis.yml:CodeQL-Build",
        "category": ".github/workflows/codeql-analysis.yml:CodeQL-Build",
        "environment": "{}",
        "state": "open",
        "commit_sha": "39406e42cb832f683daa691dd652a8dc36ee8930",
        "message": {"text": "This path depends on a user-provided value."},
        "location": {
            "path": "lib/ab12-gen.js",
            "start_line": 917,
            "end_line": 917,
            "start_column": 7,
            "end_column": 18,
        },
        "classifications": [],
    },
    "instances_url": "https://api.github.com/repos/octocat/hello-world/code-scanning/alerts/3/instances",
    "repository": {
        "id": 1296269,
        "node_id": "MDEwOlJlcG9zaXRvcnkxMjk2MjY5",
        "name": "Hello-World",
        "full_name": "octocat/Hello-World",
        "owner": {
            "login": "octocat",
            "id": 1,
            "node_id": "MDQ6VXNlcjE=",
            "avatar_url": "https://github.com/images/error/octocat_happy.gif",
            "gravatar_id": "",
            "url": "https://api.github.com/users/octocat",
            "html_url": "https://github.com/octocat",
            "followers_url": "https://api.github.com/users/octocat/followers",
            "following_url": "https://api.github.com/users/octocat/following{/other_user}",
            "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
            "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
            "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
            "organizations_url": "https://api.github.com/users/octocat/orgs",
            "repos_url": "https://api.github.com/users/octocat/repos",
            "events_url": "https://api.github.com/users/octocat/events{/privacy}",
            "received_events_url": "https://api.github.com/users/octocat/received_events",
            "type": "User",
            "site_admin": False,
        },
        "private": False,
        "html_url": "https://github.com/octocat/Hello-World",
        "description": "This your first repo!",
        "fork": False,
        "url": "https://api.github.com/repos/octocat/Hello-World",
        "archive_url": "https://api.github.com/repos/octocat/Hello-World/{archive_format}{/ref}",
        "assignees_url": "https://api.github.com/repos/octocat/Hello-World/assignees{/user}",
        "blobs_url": "https://api.github.com/repos/octocat/Hello-World/git/blobs{/sha}",
        "branches_url": "https://api.github.com/repos/octocat/Hello-World/branches{/branch}",
        "collaborators_url": "https://api.github.com/repos/octocat/Hello-World/collaborators{/collaborator}",
        "comments_url": "https://api.github.com/repos/octocat/Hello-World/comments{/number}",
        "commits_url": "https://api.github.com/repos/octocat/Hello-World/commits{/sha}",
        "compare_url": "https://api.github.com/repos/octocat/Hello-World/compare/{base}...{head}",
        "contents_url": "https://api.github.com/repos/octocat/Hello-World/contents/{+path}",
        "contributors_url": "https://api.github.com/repos/octocat/Hello-World/contributors",
        "deployments_url": "https://api.github.com/repos/octocat/Hello-World/deployments",
        "downloads_url": "https://api.github.com/repos/octocat/Hello-World/downloads",
        "events_url": "https://api.github.com/repos/octocat/Hello-World/events",
        "forks_url": "https://api.github.com/repos/octocat/Hello-World/forks",
        "git_commits_url": "https://api.github.com/repos/octocat/Hello-World/git/commits{/sha}",
        "git_refs_url": "https://api.github.com/repos/octocat/Hello-World/git/refs{/sha}",
        "git_tags_url": "https://api.github.com/repos/octocat/Hello-World/git/tags{/sha}",
        "issue_comment_url": "https://api.github.com/repos/octocat/Hello-World/issues/comments{/number}",
        "issue_events_url": "https://api.github.com/repos/octocat/Hello-World/issues/events{/number}",
        "issues_url": "https://api.github.com/repos/octocat/Hello-World/issues{/number}",
        "keys_url": "https://api.github.com/repos/octocat/Hello-World/keys{/key_id}",
        "labels_url": "https://api.github.com/repos/octocat/Hello-World/labels{/name}",
        "languages_url": "https://api.github.com/repos/octocat/Hello-World/languages",
        "merges_url": "https://api.github.com/repos/octocat/Hello-World/merges",
        "milestones_url": "https://api.github.com/repos/octocat/Hello-World/milestones{/number}",
        "notifications_url": "https://api.github.com/repos/octocat/Hello-World/notifications{?since,all,participating}",
        "pulls_url": "https://api.github.com/repos/octocat/Hello-World/pulls{/number}",
        "releases_url": "https://api.github.com/repos/octocat/Hello-World/releases{/id}",
        "stargazers_url": "https://api.github.com/repos/octocat/Hello-World/stargazers",
        "statuses_url": "https://api.github.com/repos/octocat/Hello-World/statuses/{sha}",
        "subscribers_url": "https://api.github.com/repos/octocat/Hello-World/subscribers",
        "subscription_url": "https://api.github.com/repos/octocat/Hello-World/subscription",
        "tags_url": "https://api.github.com/repos/octocat/Hello-World/tags",
        "teams_url": "https://api.github.com/repos/octocat/Hello-World/teams",
        "trees_url": "https://api.github.com/repos/octocat/Hello-World/git/trees{/sha}",
        "hooks_url": "https://api.github.com/repos/octocat/Hello-World/hooks",
    },
}


class RuleTestCase(unittest.TestCase):
    def test_from_dict(self):
        rule = Rule.from_dict(
            {
                "id": "js/zipslip",
                "severity": "error",
                "tags": ["security", "external/cwe/cwe-022"],
                "description": "Arbitrary file write during zip extraction",
                "name": "js/zipslip",
            }
        )

        self.assertEqual(rule.id, "js/zipslip")
        self.assertEqual(rule.name, "js/zipslip")
        self.assertEqual(rule.severity, Severity.ERROR)
        self.assertEqual(len(rule.tags), 2)
        self.assertEqual(rule.tags, ["security", "external/cwe/cwe-022"])
        self.assertEqual(
            rule.description, "Arbitrary file write during zip extraction"
        )


class LocationTestCase(unittest.TestCase):
    def test_from_dict(self):
        location = Location.from_dict(
            {
                "path": "lib/ab12-gen.js",
                "start_line": 917,
                "end_line": 917,
                "start_column": 7,
                "end_column": 18,
            }
        )

        self.assertEqual(location.path, "lib/ab12-gen.js")
        self.assertEqual(location.start_line, 917)
        self.assertEqual(location.end_line, 917)
        self.assertEqual(location.start_column, 7)
        self.assertEqual(location.end_column, 18)


class InstanceTestCase(unittest.TestCase):
    def test_from_dict(self):
        instance = Instance.from_dict(
            {
                "ref": "refs/heads/main",
                "analysis_key": ".github/workflows/codeql-analysis.yml:CodeQL-Build",
                "category": ".github/workflows/codeql-analysis.yml:CodeQL-Build",
                "environment": "{}",
                "state": "open",
                "commit_sha": "39406e42cb832f683daa691dd652a8dc36ee8930",
                "message": {
                    "text": "This path depends on a user-provided value."
                },
                "location": {
                    "path": "lib/ab12-gen.js",
                    "start_line": 917,
                    "end_line": 917,
                    "start_column": 7,
                    "end_column": 18,
                },
                "classifications": [],
            }
        )

        self.assertEqual(instance.ref, "refs/heads/main")
        self.assertEqual(
            instance.analysis_key,
            ".github/workflows/codeql-analysis.yml:CodeQL-Build",
        )
        self.assertEqual(
            instance.category,
            ".github/workflows/codeql-analysis.yml:CodeQL-Build",
        )
        self.assertEqual(instance.environment, "{}")
        self.assertEqual(instance.state, AlertState.OPEN)
        self.assertEqual(
            instance.commit_sha, "39406e42cb832f683daa691dd652a8dc36ee8930"
        )
        self.assertEqual(
            instance.message.text, "This path depends on a user-provided value."
        )

        self.assertEqual(instance.location.path, "lib/ab12-gen.js")
        self.assertEqual(instance.location.start_line, 917)
        self.assertEqual(instance.location.end_line, 917)
        self.assertEqual(instance.location.start_column, 7)
        self.assertEqual(instance.location.end_column, 18)

        self.assertEqual(len(instance.classifications), 0)


class ToolTestCase(unittest.TestCase):
    def test_from_dict(self):
        tool = Tool.from_dict(
            {"name": "CodeQL", "guid": None, "version": "2.4.0"}
        )

        self.assertEqual(tool.name, "CodeQL")
        self.assertEqual(tool.version, "2.4.0")
        self.assertIsNone(tool.guid)


class CodeScanningAlertTestCase(unittest.TestCase):
    def test_from_dict(self):
        alert = CodeScanningAlert.from_dict(ALERT)

        self.assertEqual(alert.number, 3)
        self.assertEqual(
            alert.created_at,
            datetime(2020, 2, 13, 12, 29, 18, tzinfo=timezone.utc),
        )
        self.assertEqual(
            alert.url,
            "https://api.github.com/repos/octocat/hello-world/code-scanning/alerts/3",
        )
        self.assertEqual(
            alert.html_url,
            "https://github.com/octocat/hello-world/code-scanning/3",
        )
        self.assertEqual(alert.state, AlertState.DISMISSED)
        self.assertEqual(
            alert.dismissed_at,
            datetime(2020, 2, 14, 12, 29, 18, tzinfo=timezone.utc),
        )
        self.assertEqual(alert.dismissed_by.login, "octocat")
        self.assertEqual(
            alert.dismissed_comment,
            "This alert is not actually correct, because there's a sanitizer "
            "included in the library.",
        )
        self.assertEqual(alert.rule.id, "js/zipslip")
        self.assertEqual(alert.tool.name, "CodeQL")
        self.assertEqual(alert.most_recent_instance.ref, "refs/heads/main")
        self.assertEqual(
            alert.most_recent_instance.location.path, "lib/ab12-gen.js"
        )
        self.assertEqual(
            alert.instances_url,
            "https://api.github.com/repos/octocat/hello-world/code-scanning/alerts/3/instances",
        )
        self.assertEqual(alert.repository.id, 1296269)


class AnalysisTestCase(unittest.TestCase):
    def test_from_dict(self):
        analysis = Analysis.from_dict(
            {
                "ref": "refs/heads/main",
                "commit_sha": "d99612c3e1f2970085cfbaeadf8f010ef69bad83",
                "analysis_key": ".github/workflows/codeql-analysis.yml:analyze",
                "environment": '{"language":"python"}',
                "error": "",
                "category": ".github/workflows/codeql-analysis.yml:analyze/language:python",
                "created_at": "2020-08-27T15:05:21Z",
                "results_count": 17,
                "rules_count": 49,
                "id": 201,
                "url": "https://api.github.com/repos/octocat/hello-world/code-scanning/analyses/201",
                "sarif_id": "6c81cd8e-b078-4ac3-a3be-1dad7dbd0b53",
                "tool": {"name": "CodeQL", "guid": None, "version": "2.4.0"},
                "deletable": True,
                "warning": "",
            }
        )

        self.assertEqual(analysis.ref, "refs/heads/main")
        self.assertEqual(
            analysis.commit_sha, "d99612c3e1f2970085cfbaeadf8f010ef69bad83"
        )
        self.assertEqual(
            analysis.analysis_key,
            ".github/workflows/codeql-analysis.yml:analyze",
        )
        self.assertEqual(analysis.environment, '{"language":"python"}')
        self.assertEqual(analysis.error, "")
        self.assertEqual(analysis.warning, "")
        self.assertEqual(
            analysis.category,
            ".github/workflows/codeql-analysis.yml:analyze/language:python",
        )
        self.assertEqual(
            analysis.created_at,
            datetime(2020, 8, 27, 15, 5, 21, tzinfo=timezone.utc),
        )
        self.assertEqual(analysis.results_count, 17)
        self.assertEqual(analysis.rules_count, 49)
        self.assertEqual(analysis.id, 201)
        self.assertEqual(
            analysis.url,
            "https://api.github.com/repos/octocat/hello-world/code-scanning/analyses/201",
        )
        self.assertEqual(
            analysis.sarif_id, "6c81cd8e-b078-4ac3-a3be-1dad7dbd0b53"
        )
        self.assertEqual(analysis.tool.name, "CodeQL")
        self.assertEqual(analysis.tool.version, "2.4.0")
        self.assertIsNone(analysis.tool.guid)
        self.assertTrue(analysis.deletable)


class CodeQLDatabaseTestCase(unittest.TestCase):
    def test_from_dict(self):
        db = CodeQLDatabase.from_dict(
            {
                "id": 1,
                "name": "database.zip",
                "language": "java",
                "uploader": {
                    "login": "octocat",
                    "id": 1,
                    "node_id": "MDQ6VXNlcjE=",
                    "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                    "gravatar_id": "",
                    "url": "https://api.github.com/users/octocat",
                    "html_url": "https://github.com/octocat",
                    "followers_url": "https://api.github.com/users/octocat/followers",
                    "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                    "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                    "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                    "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                    "organizations_url": "https://api.github.com/users/octocat/orgs",
                    "repos_url": "https://api.github.com/users/octocat/repos",
                    "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                    "received_events_url": "https://api.github.com/users/octocat/received_events",
                    "type": "User",
                    "site_admin": False,
                },
                "content_type": "application/zip",
                "size": 1024,
                "created_at": "2022-09-12T12:14:32Z",
                "updated_at": "2022-09-12T12:14:32Z",
                "url": "https://api.github.com/repos/octocat/Hello-World/code-scanning/codeql/databases/java",
                "commit_oid": "12345678901234567000",
            }
        )

        self.assertEqual(db.id, 1)
        self.assertEqual(db.name, "database.zip")
        self.assertEqual(db.language, "java")
        self.assertEqual(db.uploader.id, 1)
        self.assertEqual(db.content_type, "application/zip")
        self.assertEqual(db.size, 1024)
        self.assertEqual(
            db.created_at,
            datetime(2022, 9, 12, 12, 14, 32, tzinfo=timezone.utc),
        )
        self.assertEqual(
            db.updated_at,
            datetime(2022, 9, 12, 12, 14, 32, tzinfo=timezone.utc),
        )
        self.assertEqual(
            db.url,
            "https://api.github.com/repos/octocat/Hello-World/code-scanning/codeql/databases/java",
        )
        self.assertEqual(db.commit_oid, "12345678901234567000")


class DefaultSetupTestCase(unittest.TestCase):
    def test_from_dict(self):
        setup = DefaultSetup.from_dict(
            {
                "state": "configured",
                "languages": ["ruby", "python"],
                "query_suite": "default",
                "updated_at": "2023-01-19T11:21:34Z",
                "schedule": "weekly",
            }
        )

        self.assertEqual(setup.state, DefaultSetupState.CONFIGURED)
        self.assertEqual(setup.languages, [Language.RUBY, Language.PYTHON])
        self.assertEqual(setup.query_suite, QuerySuite.DEFAULT)
        self.assertEqual(
            setup.updated_at,
            datetime(2023, 1, 19, 11, 21, 34, tzinfo=timezone.utc),
        )
        self.assertEqual(setup.schedule, "weekly")


class SarifUploadInformationTestCase(unittest.TestCase):
    def test_from_dict(self):
        sarif = SarifUploadInformation.from_dict(
            {
                "processing_status": "complete",
                "analyses_url": "https://api.github.com/repos/octocat/hello-world/code-scanning/analyses?sarif_id=47177e22-5596-11eb-80a1-c1e54ef945c6",
            }
        )
        self.assertEqual(
            sarif.processing_status, SarifProcessingStatus.COMPLETE
        )
        self.assertEqual(
            sarif.analyses_url,
            "https://api.github.com/repos/octocat/hello-world/code-scanning/analyses?sarif_id=47177e22-5596-11eb-80a1-c1e54ef945c6",
        )
        self.assertIsNone(sarif.errors)
